import json
import os
import pytest
import sh
import shutil
import yaml

from enough import settings
from enough.common import Enough
from enough.common.openstack import OpenStackBase, OpenStack

from tests import prepare_config_dir


@pytest.mark.openstack_integration
def test_clone_and_destroy(request, tmpdir):

    clone_override_dir = f'{tmpdir}/clone-override'
    shutil.copytree('tests/enough/common/test_init/clone-override',
                    clone_override_dir)
    original_domain = 'original.com'

    # TODO: swap the two instructions below once Python 3.8 is used
    # and update copytree parameters:
    # 1. add dirs_exist_ok=True parameter
    # 2. use original_config_dir instead of f'{tmpdir}/{original_domain}
    shutil.copytree('tests/enough/common/test_init/backup',
                    f'{tmpdir}/{original_domain}')
    original_config_dir = prepare_config_dir(original_domain, tmpdir)

    password = 'PASSWORD'
    open(f'{original_config_dir}.pass', 'w').write(password)

    original_all_dir = f'{original_config_dir}/inventory/group_vars/all'
    assert not os.path.exists(f'{original_all_dir}/internal_network.yml')

    original = Enough(original_config_dir, settings.SHARE_DIR,
                      domain=original_domain,
                      driver='openstack')
    assert original.openstack.o.server.list().strip() == ''

    clone_domain = 'clone.com'
    clone = original.clone(clone_domain, 'clone', True)

    assert password == open(f'{clone.config_dir}.pass').read()
    assert 'REPLACED CONTENT' == open(
        f'{clone.config_dir}/inventory/host_vars/to-replace.yml').read().strip()
    assert clone.openstack.o.server.list().strip() == ''
    clone_all_dir = f'{clone.config_dir}/inventory/group_vars/all'
    assert os.path.exists(f'{clone_all_dir}/internal_network.yml')

    assert os.path.exists(clone.config_dir)
    clone.destroy()
    assert not os.path.exists(clone.config_dir)


def create_enough(tmpdir, dotenough, **kwargs):

    enough_domain = 'enough.com'

    # TODO: swap the two instructions below once Python 3.8 is used
    # and update copytree parameters:
    # 1. add dirs_exist_ok=True parameter
    # 2. use config_dir instead of f'{tmpdir}/{enough_domain}
    shutil.copytree(f'tests/enough/common/test_init/{dotenough}',
                    f'{tmpdir}/{enough_domain}')
    config_dir = prepare_config_dir(enough_domain, tmpdir)

    enough = Enough(config_dir, settings.SHARE_DIR,
                    domain=enough_domain,
                    driver='openstack',
                    **kwargs)
    return enough


def test_clone_clobber(request, tmpdir):
    original = create_enough(tmpdir, 'copy')
    shutil.copyfile('tests/enough/common/test_init/clone-clobber/clouds.yml',
                    f'{original.config_dir}/inventory/group_vars/all/clouds.yml')
    clone_domain = 'clone.com'
    clone = original.clone(clone_domain, 'clone', False)
    assert os.path.exists(f'{clone.config_dir}')
    stone = f'{clone.config_dir}/STONE'
    open(stone, 'w').write('INSIDE')

    assert os.path.exists(stone)
    clone = original.clone(clone_domain, 'clone', False)
    assert os.path.exists(stone)
    clone = original.clone(clone_domain, 'clone', True)
    assert not os.path.exists(stone)


def create_and_clone_server_and_volume(tmpdir):
    original = create_enough(tmpdir, 'backup')
    original.set_args(name='sample', playbook='enough-playbook.yml')
    original.service.create_or_update()

    clone_domain = 'clone.com'
    clone = original.clone(clone_domain, 'clone', True)
    clone.set_args(name='sample', playbook='enough-playbook.yml')
    clone.service.create_or_update()

    return (original, clone)


@pytest.mark.openstack_integration
def test_clone_create_service(request, tmpdir):
    try:
        (original, clone) = create_and_clone_server_and_volume(tmpdir)
        assert 'sample-host' in original.openstack.o.server.list()
        assert 'sample-volume' in original.openstack.o.volume.list()
        assert 'sample-host' in clone.openstack.o.server.list()
        assert 'sample-volume' in clone.openstack.o.volume.list()
    finally:
        for cloud in ('production', 'clone'):
            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
            # comment out the following line to re-use the content of the regions and save time
            o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_create_copy_host(request, tmpdir):
    try:
        enough = create_enough(tmpdir, 'copy')
        if 'copy-volume' not in enough.openstack.o.volume.list():
            enough.openstack.o.volume.create('--size', '1', 'copy-volume')
        ip = enough.create_copy_host('copy-from-host', 'some-volume', 'copy-volume')
        sh.ssh_keygen('-R', ip, _ok_code=(0, 255))
        r = sh.ssh('-oStrictHostKeyChecking=no',
                   '-i', enough.dotenough.private_key(), f'root@{ip}', 'id')
        assert 'uid=0(root)' in r
        r = sh.ssh('-oStrictHostKeyChecking=no',
                   '-i', enough.dotenough.private_key(), f'root@{ip}', 'mountpoint', '/srv').strip()
        assert r == '/srv is a mountpoint'
    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_create_volume_from_snapshot(request, tmpdir):
    try:
        e = create_enough(tmpdir, 'backup')
        e.set_args(name='sample', playbook='enough-playbook.yml')
        e.service.create_or_update()
        e.openstack.backup_create(['sample-volume'])
        snapshot = f'{e.openstack.backup_date()}-sample-volume'
        assert snapshot in e.openstack.o.volume.snapshot.list()

        volume = 'test-volume'
        e.create_volume_from_snapshot(snapshot, volume)

        assert volume in e.openstack.o.volume.list()

    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_clone_volume_from_snapshot(request, tmpdir):
    try:
        original = create_enough(tmpdir, 'backup')
        original.set_args(name='sample', playbook='enough-playbook.yml')
        original.service.create_or_update()
        ip = original.hosts.load().get_ip('sample-host')
        sh.ssh_keygen('-R', ip, _ok_code=(0, 255))
        sh.ssh('-oStrictHostKeyChecking=no',
               '-i', original.dotenough.private_key(), f'debian@{ip}', 'touch', '/srv/STONE')
        sh.ssh('-oStrictHostKeyChecking=no',
               '-i', original.dotenough.private_key(), f'debian@{ip}', 'sync')
        original.openstack.backup_create(['sample-volume'])
        snapshot = f'{original.openstack.backup_date()}-sample-volume'
        assert snapshot in original.openstack.o.volume.snapshot.list()

        clone_domain = 'clone.com'
        clone = original.clone(clone_domain, 'clone', True)

        (from_ip, to_ip, from_volume, to_volume) = original._clone_volume_from_snapshot_body(
            clone, snapshot)

        sh.ssh_keygen('-R', from_ip, _ok_code=(0, 255))
        assert sh.ssh('-oStrictHostKeyChecking=no',
                      '-i', original.dotenough.private_key(), f'root@{from_ip}',
                      'test', '-e', '/srv/STONE').exit_code == 0
        sh.ssh_keygen('-R', to_ip, _ok_code=(0, 255))
        assert sh.ssh('-oStrictHostKeyChecking=no',
                      '-i', clone.dotenough.private_key(), f'root@{to_ip}',
                      'test', '!', '-e', '/srv/STONE').exit_code == 0
        original._rsync_copy_host(from_ip, to_ip)
        assert sh.ssh('-oStrictHostKeyChecking=no',
                      '-i', clone.dotenough.private_key(), f'root@{to_ip}',
                      'test', '-e', '/srv/STONE').exit_code == 0

        original._clone_volume_from_snapshot_cleanup(clone, from_volume)

    finally:
        for cloud in ('production', 'clone'):
            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
            # comment out the following line to re-use the content of the regions and save time
            o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_create_service_matching_snapshot(request, tmpdir):
    try:
        enough = create_enough(tmpdir, 'backup')
        host = enough.create_service_matching_snapshot('2020-02-20-sample-volume')
        assert host == 'sample-host'
        assert 'sample-volume' in enough.openstack.o.volume.list()

    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_restore_remote(request, tmpdir):
    try:
        original = create_enough(tmpdir, 'backup')
        original.set_args(name='sample', playbook='enough-playbook.yml')
        original.service.create_or_update()
        hosts = original.hosts.load()
        ip = hosts.get_ip('sample-host')
        port = hosts.get_port('sample-host')

        sh.ssh_keygen('-R', ip, _ok_code=(0, 255))
        sh.ssh('-oStrictHostKeyChecking=no',
               '-i', original.dotenough.private_key(), f'-p{port}',
               f'debian@{ip}', 'touch', '/srv/STONE')
        original.openstack.backup_create(['sample-volume'])
        snapshot = f'{original.openstack.backup_date()}-sample-volume'
        assert snapshot in original.openstack.o.volume.snapshot.list()

        # original host isn't needed anymore
        original.openstack.o.server.stop('sample-host')
        original.openstack.server_wait_shutoff('sample-host')

        clone = original.restore_remote('test.com', 'clone', True, snapshot)
        assert 'sample-volume' in clone.openstack.o.volume.list()
        clone_hosts = clone.hosts.load()
        clone_ip = clone_hosts.get_ip('sample-host')
        clone_port = clone_hosts.get_port('sample-host')

        assert ip != clone_ip

        OpenStackBase.wait_for_ssh(clone_ip, clone_port)
        sh.ssh_keygen('-R', clone_ip, _ok_code=(0, 255))
        res = sh.ssh('-oStrictHostKeyChecking=no',
                     '-i', clone.dotenough.private_key(), f'-p{clone_port}',
                     f'debian@{clone_ip}',
                     'test', '-e', '/srv/STONE')
        assert res.exit_code == 0, f'Unexpected SSH exit code: {res.exit_code}'

    finally:
        for cloud in ('production', 'clone'):
            o = OpenStack(settings.CONFIG_DIR, cloud=cloud)
            # comment out the following line to re-use the content of the regions and save time
            o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_restore_local(request, tmpdir):
    try:
        e = create_enough(tmpdir, 'backup')
        e.set_args(name='sample', playbook='enough-playbook.yml')
        e.service.create_or_update()
        hosts = e.hosts.load()
        ip = hosts.get_ip('sample-host')
        port = hosts.get_port('sample-host')

        sh.ssh_keygen('-R', ip, _ok_code=(0, 255))
        sh.ssh('-oStrictHostKeyChecking=no', '-i', e.dotenough.private_key(),
               f'-p{port}', f'debian@{ip}', 'touch', '/srv/STONE')
        e.openstack.backup_create(['sample-volume'])
        sh.ssh('-oStrictHostKeyChecking=no', '-i', e.dotenough.private_key(),
               f'-p{port}', f'debian@{ip}', 'rm', '/srv/STONE')
        snapshot = f'{e.openstack.backup_date()}-sample-volume'
        assert snapshot in e.openstack.o.volume.snapshot.list()

        assert e == e.restore_local(snapshot)
        assert 'sample-volume' in e.openstack.o.volume.list()
        OpenStackBase.wait_for_ssh(ip, port)
        sh.ssh_keygen('-R', ip, _ok_code=(0, 255))
        res = sh.ssh('-oStrictHostKeyChecking=no', '-i', e.dotenough.private_key(),
                     f'-p{port}', f'debian@{ip}', 'test', '-e', '/srv/STONE')
        assert res.exit_code == 0, f'Unexpected SSH exit code: {res.exit_code}'

    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


@pytest.mark.openstack_integration
def test_create_missings(request, tmpdir):
    try:
        enough = create_enough(tmpdir, 'create-missings')
        r = enough.create_missings(['bind-host'])
        assert 'bind-host' in r
        internal_dns = enough.openstack.o.subnet.show(
            '--format=json', '-c', 'dns_nameservers', 'internal')
        internal_dns = json.loads(internal_dns.stdout)
        assert len(internal_dns['dns_nameservers']) == 1
        bind_internal_ip = enough.openstack.server_ip_in_network('bind-host', 'internal')
        assert bind_internal_ip == internal_dns['dns_nameservers'][0]
    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


def test_replace_production(tmpdir):
    target_cloud = 'clone'
    clouds = {
        'clouds': {
            target_cloud: 'CLONE',
            'production': 'PRODUCTION',
        }
    }
    clouds_path = f'{tmpdir}/clouds.yml'
    open(clouds_path, 'w').write(yaml.dump(clouds))
    Enough.replace_production(clouds_path, target_cloud)
    clouds = yaml.safe_load(open(clouds_path))
    assert clouds['clouds']['production'] == 'CLONE'
    assert target_cloud not in clouds['clouds']


def test_volume_from_snapshot():
    volume = 'NAME'
    snapshot = f'2010-07-08-{volume}'
    assert Enough.volume_from_snapshot(snapshot) == volume


def test_host_from_snapshot(request, tmpdir):
    e = create_enough(tmpdir, 'backup')
    host = 'sample-host'
    snapshot = f'2010-07-08-sample-volume'
    assert e.host_from_snapshot(snapshot) == host


@pytest.mark.openstack_integration
def test_volume_resize(request, tmpdir):
    try:
        enough = create_enough(tmpdir, 'resize')
        enough.set_args(name='sample', playbook='enough-playbook.yml')
        enough.service.create_or_update()
        # False because the size does not change
        assert enough.volume_resize('sample-host', 'sample-volume') is False
        with pytest.raises(Enough.VolumeResizeUndefined):
            enough.volume_resize('other-host', 'other-volume')
        with pytest.raises(Enough.VolumeResizeNoSize):
            enough.volume_resize('nosize-host', 'nosize-volume')
    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)


#
# Host
#
@pytest.mark.openstack_integration
def test_host_create_or_update(request, tmpdir, openstack_name):
    try:
        enough = create_enough(tmpdir, 'host')
        enough.set_args(name='new-host')
        info = enough.host.create_or_update()
        assert 'ipv4' in info
    finally:
        o = OpenStack(settings.CONFIG_DIR)
        # comment out the following line to re-use the content of the regions and save time
        o.destroy_everything(None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019,      Maciej Delmanowski <drybjed@gmail.com>
# Copyright: (c) 2019,      DebOps https://debops.org/
# Copyright: (c) 2017-2018, Yann Amar <quidame@poivron.org>
#
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# Source: https://github.com/quidame/ansible-module-dpkg_divert

from __future__ import absolute_import, division, print_function
from ansible.module_utils.basic import AnsibleModule
import os.path
import errno
import os
import re

__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = '''
---
module: dpkg_divert
short_description: Override a package's version of a file
description:
    - A diversion is for C(dpkg) the knowledge that only a given I(package)
      is allowed to install a file at a given I(path). Other packages shipping
      their own version of this file will be forced to I(divert) it, i.e. to
      install it at another location. It allows one to keep changes in a file
      provided by a debian package by preventing its overwrite at package
      upgrade.
    - This module manages diversions of debian packages files using the
      C(dpkg-divert)(1) commandline tool. It can either create or remove a
      diversion for a given file, but also update an existing diversion to
      modify its holder and/or its divert path.
    - It's a feature of this module to mimic C(dpkg-divert)'s behaviour
      regarding the renaming of files when removing as well as adding a
      diversion: existing files are never overwritten.
version_added: "2.4"
author: "quidame@poivron.org"
options:
    path:
        description:
            - The original and absolute path of the file to be diverted or
              undiverted. This path is unique, i.e. it is not possible to get
              two diversions for the same I(path).
        required: true
        type: 'path'
        aliases: [ 'name' ]
    state:
        description:
            - When I(state=absent), remove the diversion of the specified
              I(path); when I(state=present), create the diversion if it does
              not exist, or update its I(package) holder or I(divert) path,
              if any, and if I(force) is C(True).
            - Unless I(force) is C(True), the removal of I(path)'s diversion
              only happens if the diversion matches the I(divert) and
              I(package) values, if any.
        type: 'string'
        default: 'present'
        choices: [ 'absent', 'present' ]
    package:
        description:
            - The name of the package whose copy of file is not diverted, also
              known as the diversion holder or the package the diversion
              belongs to.
            - The actual package does not have to be installed or even to exist
              for its name to be valid. If not specified, the diversion is hold
              by 'LOCAL', that is reserved by/for dpkg for local dversions.
            - Removing or updating a diversion fails if the diversion exists
              and belongs to another package, unless I(force) is C(True).
    divert:
        description:
            - The location where the versions of file will be diverted.
            - Default is to add suffix C(.distrib) to the file path.
        type: 'path'
    rename:
        description:
            - Actually move the file aside (or back).
            - Renaming is skipped (but module doesn't fail) in case the
              destination file already exists. This is a C(dpkg-divert)
              feature, and its purpose is to never overwrite a file. It also
              makes the command itself idempotent, and the module's I(force)
              parameter has no effect on this behaviour.
            - Also, I(rename) is ignored if the diversion entry is unchanged
              in the diversion database (adding an already existing diversion
              or removing a non-existing one).
        type: 'bool'
        default: true
    delete:
        description:
            - When I(yes), delete the file in place of the original before
              reverting. This only applies with I(state=absent) to avoid
              C(dpkg-divert) command complaining about existing file in place
              of the diverted one.
        type: 'bool'
        default: false
    force:
        description:
            - Force to divert file when diversion already exists and is hold
              by another I(package) or points to another I(divert). There is
              no need to use it for I(remove) action if I(divert) or I(package)
              are not used.
            - This doesn't override the rename's lock feature, i.e. it doesn't
              help to force I(rename), but only to force the diversion for
              dpkg.
        type: 'bool'
        default: false
requirements: [ dpkg-divert, env ]
'''

EXAMPLES = '''
# Divert /etc/screenrc to /etc/screenrc.dpkg-divert and rename the file
- name: Create local diversion
  dpkg_divert: path=/etc/screenrc

# Divert /etc/screenrc to /etc/screenrc.distrib for package 'branding' and
# rename the file
- name: Create diversion for APT package
  dpkg_divert:
    name: /etc/screenrc
    package: branding

- name: Delete the file in place of the original and remove the diversion
  dpkg_divert:
    name: /etc/screenrc
    state: absent
    delete: yes

- name: remove the screenrc diversion only if belonging to 'branding'
  dpkg_divert:
    name: /etc/screenrc
    package: branding
    state: absent

# Divert screenrc to screenrc.dpkg-divert, but don't rename the file
- name: Divert with custom rename
  dpkg_divert:
    path: /etc/screenrc
    divert: /etc/screenrc.dpkg-divert
    rename: no

# Divert and rename screenrc to screenrc.dpkg-divert, even if diversion is
# already set
- name: Divert with custom rename
  dpkg_divert:
    path: /etc/screenrc
    divert: /etc/screenrc.dpkg-divert
    rename: yes
    force: yes

  # Remove the screenrc diversion and maybe move the diverted file to its
  # original place
- name: Remove diversion and rename file
  dpkg_divert:
    path: /etc/screenrc
    state: absent
    rename: yes
'''


def main():

    # Mimic the behaviour of the dpkg-divert(1) command: '--add' is implicit
    # when not using '--remove'; '--rename' takes care to never overwrite
    # existing files; and options are intended to not conflict between them.

    # 'force' is an option of the module, not of the command, and implies to
    # run the command twice. Its purpose is to allow one to re-divert a file
    # with another target path or to 'give' it to another package, in one task.
    # This is very easy because one of the values is unique in the diversion
    # database, and dpkg-divert itself is idempotent (does nothing when nothing
    # needs doing).

    module = AnsibleModule(
        argument_spec=dict(
            path=dict(required=True,  type='path', aliases=['name']),
            state=dict(required=False, type='str', default='present',
                       choices=['absent', 'present']),
            package=dict(required=False, type='str', default='LOCAL'),
            divert=dict(required=False, type='path'),
            rename=dict(required=False, type='bool', default=True),
            delete=dict(required=False, type='bool', default=False),
            force=dict(required=False, type='bool', default=False),
        ),
        supports_check_mode=True,
    )

    path = module.params['path']
    state = module.params['state']
    package = module.params['package']
    divert = module.params['divert']
    rename = module.params['rename']
    delete = module.params['delete']
    force = module.params['force']

    DPKG_DIVERT = module.get_bin_path('dpkg-divert', required=True)
    # We need to parse the command's output, which is localized.
    # So we have to reset environment variable (LC_ALL).
    ENVIRONMENT = module.get_bin_path('env', required=True)

    # Start to build the commandline we'll have to run
    COMMANDLINE = [ENVIRONMENT, 'LC_ALL=C', DPKG_DIVERT, path]

    # Then insert options as requested in the task parameters:
    if state == 'absent':
        COMMANDLINE.insert(3, '--remove')
    elif state == 'present':
        COMMANDLINE.insert(3, '--add')

    if rename:
        COMMANDLINE.insert(3, '--rename')

    if divert:
        COMMANDLINE.insert(3, '--divert')
        COMMANDLINE.insert(4, divert)
    else:
        if package == 'LOCAL':
            COMMANDLINE.insert(3, '--divert')
            COMMANDLINE.insert(4, '.'.join([path, 'dpkg-divert']))
        elif package:
            COMMANDLINE.insert(3, '--divert')
            COMMANDLINE.insert(4, '.'.join([path, 'distrib']))

    if package == 'LOCAL':
        COMMANDLINE.insert(3, '--local')
    elif package:
        COMMANDLINE.insert(3, '--package')
        COMMANDLINE.insert(4, package)

    # dpkg-divert has a useful --test option that we will use in check mode or
    # when needing to parse output before actually doing anything.
    TESTCOMMAND = list(COMMANDLINE)
    TESTCOMMAND.insert(3, '--test')
    if module.check_mode:
        COMMANDLINE = list(TESTCOMMAND)

    cmd = ' '.join(COMMANDLINE)

    # `dpkg-divert --listpackage FILE` always returns 0, but not diverted files
    # provide no output.
    rc, listpackage, _ = module.run_command(
            [DPKG_DIVERT, '--listpackage', path])
    rc, placeholder, _ = module.run_command(TESTCOMMAND)

    # There is probably no need to do more than that. Please read the first
    # sentence of the next comment for a better understanding of the following
    # `if` statement:
    if rc == 0 or not force or not listpackage:

        # If requested, delete the file to make way for the reverted one, but
        # only of the diversion currently exists.
        if not module.check_mode:
            if state == 'absent' and listpackage and delete:
                try:
                    os.unlink(path)
                except OSError as e:
                    # It may already have been removed
                    if e.errno != errno.ENOENT:
                        raise AnsibleModuleError(
                                results={'msg': "unlinking failed: %s "
                                         % to_native(e), 'path': path})

        # In the check mode, the 'dpkg-divert' command still tests the
        # diversion removal for real and returns with an error when a changed
        # file is in place. In that specific case, we instead simulate a file
        # deletion and diversion removal ourselves to have the check mode
        # succeed.
        if (module.check_mode and state == 'absent' and delete and
                listpackage and os.path.exists(path)):
            fake_stdout = ['Deleting', path, 'and', 'removing']
            if package == 'LOCAL':
                fake_stdout.append('local')
            fake_stdout.extend(['diversion', 'of', path, 'to'])
            if divert:
                fake_stdout.append(divert)
            else:
                if package == 'LOCAL':
                    fake_stdout.append('.'.join([path, 'dpkg-divert']))
                elif package:
                    fake_stdout.append('.'.join([path, 'distrib']))

            rc, stdout, stderr = [0, ' '.join(fake_stdout), '']
        else:
            rc, stdout, stderr = module.run_command(COMMANDLINE, check_rc=True)

        if re.match('^(Leaving|No diversion)', stdout):
            module.exit_json(changed=False, stdout=stdout,
                             stderr=stderr, cmd=cmd)
        else:
            module.exit_json(changed=True, stdout=stdout,
                             stderr=stderr, cmd=cmd)

    # So, here we are: the test failed AND force is true AND a diversion exists
    # for the file.  Anyway, we have to remove it first (then stop here, or add
    # a new diversion for the same file), and without failure. Cases of failure
    # with dpkg-divert are:
    # - The diversion does not belong to the same package (or LOCAL)
    # - The divert filename is not the same (e.g. path.distrib != path.divert)
    # So: force removal by stripping '--package' and '--divert' options... and
    # their arguments. Fortunately, this module accepts only a few parameters,
    # so we can rebuild a whole command line from scratch at no cost:
    FORCEREMOVE = [ENVIRONMENT, 'LC_ALL=C', DPKG_DIVERT, '--remove', path]
    module.check_mode and FORCEREMOVE.insert(3, '--test')
    rename and FORCEREMOVE.insert(3, '--rename')
    forcerm = ' '.join(FORCEREMOVE)

    if state == 'absent':
        rc, stdout, stderr = module.run_command(FORCEREMOVE, check_rc=True)
        module.exit_json(changed=True, stdout=stdout,
                         stderr=stderr, cmd=forcerm)

    # The situation is that we want to modify the settings (package or divert)
    # of an existing diversion. dpkg-divert does not handle this, and we have
    # to remove the diversion and set a new one. First, get state info:
    rc, truename, _ = module.run_command([DPKG_DIVERT, '--truename', path])
    rc, rmout, rmerr = module.run_command(FORCEREMOVE, check_rc=True)
    if module.check_mode:
        module.exit_json(changed=True, cmd=[forcerm, cmd],
                         msg=[rmout,
                              "*** RUNNING IN CHECK MODE ***",
                              "The next step can't be actually performed - "
                              "even dry-run - without error (since the "
                              "previous removal didn't happen) but is "
                              "supposed to achieve the task."])

    old = truename.rstrip()
    if divert:
        new = divert
    else:
        if package == 'LOCAL':
            new = '.'.join([path, 'dpkg-divert'])
        elif package:
            new = '.'.join([path, 'distrib'])

    # Store state of files as they may change
    old_exists = os.path.isfile(old)
    new_exists = os.path.isfile(new)

    # RENAMING NOT REMAINING
    # The behaviour of this module is to NEVER overwrite a file, i.e. never
    # change file contents but only file paths and only if not conflicting,
    # as does dpkg-divert. It means that if there is already a diversion for
    # a given file and the divert file exists too, the divert file must be
    # moved from old to new divert paths between the two dpkg-divert commands,
    # because:
    #
    # src = /etc/screenrc           (tweaked ; exists)
    # old = /etc/screentc.distrib   (default ; exists)
    # new = /etc/screenrc.ansible   (not existing yet)
    #
    # Without extra move:
    # 1. dpkg-divert --rename --remove src
    #    => dont move old to src because src exists
    # 2. dpkg-divert --rename --divert new --add src
    #    => move src to new because new doesn't exist
    # Results:
    #   - old still exists with default contents
    #   - new holds the tweaked contents
    #   - src is missing
    #   => confusing, kind of breakage
    #
    # With extra move:
    # 1. dpkg-divert --rename --remove src
    #    => dont move old to src because src exists
    # 2. os.path.rename(old, new) [conditional]
    #    => move old to new because new doesn't exist
    # 3. dpkg-divert --rename --divert new --add src
    #    => dont move src to new because new exists
    # Results:
    #   - old does not exist anymore
    #   - src is still the same tweaked file
    #   - new exists with default contents
    #   => idempotency for next times, and no breakage
    #
    if rename and old_exists and not new_exists:
        os.rename(old, new)

    rc, stdout, stderr = module.run_command(COMMANDLINE)
    rc == 0 and module.exit_json(changed=True, stdout=stdout, stderr=stderr,
                                 cmd=[forcerm, cmd], msg=[rmout, stdout])

    # Damn! FORCEREMOVE succeeded and COMMANDLINE failed. Try to restore old
    # state and end up with a 'failed' status anyway.
    if (rename and (old_exists and not os.path.isfile(old)) and
            (os.path.isfile(new) and not new_exists)):
        os.rename(new, old)

    RESTORE = [ENVIRONMENT, 'LC_ALL=C', DPKG_DIVERT, '--divert', old, path]
    old_pkg = listpackage.rstrip()
    if old_pkg == "LOCAL":
        RESTORE.insert(3, '--local')
    else:
        RESTORE.insert(3, '--package')
        RESTORE.insert(4, old_pkg)
    rename and RESTORE.insert(3, '--rename')

    module.run_command(RESTORE, check_rc=True)
    module.exit_json(failed=True, changed=True, stdout=stdout,
                     stderr=stderr, cmd=[forcerm, cmd])


if __name__ == '__main__':
    main()



testinfra_hosts = ['ansible://debian-host']


def test_ssh(host):
    cmd = host.run("ssh-keyscan debian-host")
    assert 0 == cmd.rc
    assert 'ssh-rsa' in cmd.stdout
    assert 'ssh-ed25519' in cmd.stdout



import json

from enough import cmd


def test_host_create(capsys, mocker, tmp_config_dir):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    r = {'port': '22', 'ipv4': '127.0.0.1'}
    mocker.patch('enough.common.host.HostOpenStack.create_or_update', return_value=r)
    assert cmd.main(['host', 'create', '--format=json', 'name']) == 0
    out, err = capsys.readouterr()
    j = json.loads(out)
    assert {'port': j['port'], 'ipv4': j['ip']} == r


def test_host_delete(mocker, tmp_config_dir):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.host.HostOpenStack.delete')
    assert cmd.main(['host', 'delete', 'name']) == 0



"""
Django settings for enough project.

Generated by 'django-admin startproject' using Django 2.1.5.

For more information on this file, see
https://docs.djangoproject.com/en/2.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.1/ref/settings/
"""

import os
from os.path import dirname, join
import sys

# Build paths inside the project like this: os.path.join(CONFIG_DIR, ...)
ENOUGH_DOMAIN = os.environ.get('ENOUGH_DOMAIN', 'enough.community')
ENOUGH_DOT = os.environ.get('ENOUGH_DOT', os.path.expanduser(f'~/.enough'))

if os.path.exists('setup.cfg'):  # running from sources
    if 'ENOUGH_DOMAIN' in os.environ:
        CONFIG_DIR = f'{ENOUGH_DOT}/{ENOUGH_DOMAIN}'
    else:
        CONFIG_DIR = os.path.abspath('.')
    SHARE_DIR = os.path.abspath('.')
else:
    CONFIG_DIR = f'{ENOUGH_DOT}/{ENOUGH_DOMAIN}'
    SHARE_DIR = f'{sys.prefix}/share/enough'

CERTS_DIR = f'{CONFIG_DIR}/certs'

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'mhh^de)&ky$ke$@z#7^zd+4&g083tldl@d7km-6)jc=cygb+07'

# SECURITY WARNING: don't run with debug turned on in production!
# DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'rest_framework',
    'rest_framework.authtoken',

    'enough.api',

    'bootstrap3',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'allauth.socialaccount.providers.gitlab',  # enabled by configure
    'enough.auth',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'enough.urls'

AUTHENTICATION_BACKENDS = (
    "allauth.account.auth_backends.AuthenticationBackend",
)

MODULE_DIR = dirname(__file__)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            join(MODULE_DIR, 'templates', 'allauth'),
            join(MODULE_DIR, 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.template.context_processors.static',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'enough.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.1/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(CONFIG_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/

STATIC_URL = '/static/'

STATICFILES_DIRS = (
    join(MODULE_DIR, "static"),
)

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 10
}

ENOUGH_GITLAB_URL = f'https://lab.{ENOUGH_DOMAIN}'

SITE_ID = 1
ACCOUNT_DEFAULT_HTTP_PROTOCOL = os.environ.get('ACCOUNT_DEFAULT_HTTP_PROTOCOL', 'https')
LOGIN_REDIRECT_URL = '/member/'
ACCOUNT_AUTHENTICATION_METHOD = 'username'
ACCOUNT_LOGOUT_ON_GET = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_EMAIL_REQUIRED = False
ACCOUNT_USERNAME_MIN_LENGTH = 1
ACCOUNT_EMAIL_VERIFICATION = 'none'  # testing...
# ACCOUNT_USER_MODEL_USERNAME_FIELD = None
SOCIALACCOUNT_AUTO_SIGNUP = False  # require social accounts to use the signup form ... I think
# For custom sign-up form:
# http://stackoverflow.com/questions/12303478/how-to-customize-user-profile-when-using-django-allauth
SOCIALACCOUNT_PROVIDERS = {
    'gitlab': {
        'GITLAB_URL': ENOUGH_GITLAB_URL,
        'SCOPE': ['read_user', 'api'],
    },
}



from django.http import HttpResponse
from django.template import loader
from django.contrib.auth.decorators import login_required

from rest_framework.authtoken.models import Token


@login_required
def member_index(request):
    t = loader.get_template('member/member-index.html')
    token = Token.objects.get_or_create(user=request.user)
    c = {'token': token[0]}
    return HttpResponse(t.render(c, request), content_type='text/html')



import logging

from cliff.command import Command
from enough import settings
from enough.common.openstack import OpenStack
from enough.common import dotenough
from enough.common import options


class Create(Command):
    "Create backups."

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument('volumes', nargs='*')
        options.set_options(parser)
        return parser

    def take_action(self, parsed_args):
        args = vars(self.app.options)
        args.update(vars(parsed_args))
        d = dotenough.DotEnoughOpenStack(settings.CONFIG_DIR, args['domain'])
        d.ensure()
        o = OpenStack(settings.CONFIG_DIR, **args)
        o.backup_create(args['volumes'])


class Prune(Command):
    "Prune old backup volumes and their snapshots."

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument('days', type=int, default=30)
        options.set_options(parser)
        return parser

    def take_action(self, parsed_args):
        args = vars(self.app.options)
        args.update(vars(parsed_args))
        d = dotenough.DotEnoughOpenStack(settings.CONFIG_DIR, args['domain'])
        d.ensure()
        o = OpenStack(settings.CONFIG_DIR, **args)
        o.backup_prune(args['days'])
        o.volume_prune(args['days'])



from cliff.command import Command

from enough import settings
from enough.common import options
from enough.common import Enough


def set_common_options(parser):
    parser.add_argument('--target-cloud',
                        default='clone',
                        help='Name of the cloud in which resources are restored')
    parser.add_argument('--target-domain',
                        help='Domain name under which the resources are restored')
    parser.add_argument('--clobber-cloud',
                        action='store_true',
                        help='Destroy all resources in --target-cloud')
    return parser


class Restore(Command):
    """Restore a service from a volume snapshot.

       If --target-domain is specified, a volume is created from the
       snapshot in the --target-cloud region. The corresponding
       service is also created in the --target-cloud region and set to
       use the volume.

       If --target-domain is not specificed, the volume used by the
       service matching the snapshot is replaced with the content of
       the snapshot.
    """

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        options.set_options(parser)
        parser.add_argument('name')
        return set_common_options(parser)

    def take_action(self, parsed_args):
        args = vars(self.app.options)
        args.update(vars(parsed_args))
        e = Enough(settings.CONFIG_DIR, settings.SHARE_DIR, **args)
        e.restore()


class CloneVolume(Command):
    "Create a volume from a volume snapshot"

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        options.set_options(parser)
        parser.add_argument('name')
        return set_common_options(parser)

    def take_action(self, parsed_args):
        args = vars(self.app.options)
        args.update(vars(parsed_args))
        e = Enough(settings.CONFIG_DIR, settings.SHARE_DIR, **args)
        e.cli_clone_volume()



#!/usr/bin/env python3

from distutils.core import setup

version = None
with open('debops/api.py') as f:
    for line in f:
        if line.startswith('__version__'):
            version = line.replace("'", '').split()[2]
            break

setup(
    name='debops-api',
    description='Machine readable metadata about the DebOps Project',
    version=version,
    author='Robin Schneider',
    author_email='ypid@riseup.net',
    url='https://github.com/debops/debops-api',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: DFSG approved'
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Documentation',
    ),
)



def set_options(parser):
    parser.add_argument('--driver', default='openstack')
    parser.add_argument('--inventory', action='append')
    o = parser.add_argument_group(title='OpenStack',
                                  description='Only when --driver=openstack')
    o.add_argument(
        '--cloud',
        default='production',
        help='Name of the cloud in which resources are provisionned')
    return parser



from tests.icinga_helper import IcingaHelper

testinfra_hosts = ['icinga-host']


class TestChecks(IcingaHelper):

    def test_host(self):
        r = self.get_client().objects.get('Host', 'jmm-host')
        assert r['attrs']['name'] == 'jmm-host'

    def test_service(self, host):
        assert self.is_service_ok('jmm-host!jmm-host')
        assert self.is_service_ok('jmm-host!jmm-host over Tor')



import jinja2
import logging
import os

from cliff.command import Command
from enough.version import __version__


class InstallScript(Command):
    "Get scripts, systemd services etc. required to run Enough."

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(InstallScript, self).get_parser(prog_name)
        parser.add_argument('--registry')
        parser.add_argument('--function', action='store_true')
        parser.add_argument('--no-version', action='store_true')
        parser.add_argument('--no-tty', action='store_true')
        parser.add_argument('path', nargs='?')
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.path:
            path = 'internal/data/install.sh'
            function = True
        else:
            path = parsed_args.path
            function = parsed_args.function
        self.args = parsed_args
        if parsed_args.registry:
            self.registry = parsed_args.registry
        else:
            self.registry = 'enoughcommunity/'
        if parsed_args.no_tty:
            self.tty = ''
        else:
            self.tty = '-ti'
        if parsed_args.no_version:
            self.version = ''
        else:
            self.version = f':{__version__}'
        i = os.path.join(os.path.dirname(__file__), '../..', path)
        content = open(i).read()
        if function:
            content = 'function enough() {' + content + '}'
        print(jinja2.Template(content).render(this=self))



from enough.common import retry


testinfra_hosts = ['ansible://debian-host']


def test_upgrade(host):
    return
    # that does not work, probably because the socket to the host needs reset after reboot
    with host.sudo():
        cmd = host.run("""
        set -x ; sed -i -e '/.*Unattended-Upgrade::Automatic-Reboot /s|^|//|' \
                 /etc/apt/apt.conf.d/50unattended-upgrades
        """)
        assert 0 == cmd.rc

        cmd = host.run("set -x ; touch /var/run/reboot-required")
        assert 0 == cmd.rc

        cmd = host.run("unattended-upgrade")
        print(cmd.stdout)
        print(cmd.stderr)
        assert 0 == cmd.rc

        @retry.retry((AssertionError, KeyError), tries=7)
        def run():
            cmd = host.run("! test -f /var/run/reboot-required")
            print(cmd.stdout)
            print(cmd.stderr)
            assert 0 == cmd.rc

        run()



import glob
import json
import logging
import os
import re
import sh
import tempfile
import textwrap

log = logging.getLogger(__name__)


class AnsibleVariableNotFound(Exception):
    pass


class Ansible(object):

    bake_method = sh.ansible

    def __init__(self, config_dir, share_dir, inventories=None):
        self.config_dir = config_dir
        self.share_dir = share_dir

        self.inventories = [
            '-i', f'{self.share_dir}/inventory',
        ]
        if self.config_dir != self.share_dir:
            self.inventories.extend(['-i', f'{self.config_dir}/inventory'])
        if inventories:
            self.inventories.extend([f'-i{i}' for i in inventories])

    def get_env(self):
        return {
            'SHARE_DIR': self.share_dir,
            'CONFIG_DIR': self.config_dir,
            'ANSIBLE_NOCOLOR': 'true',
            'ANSIBLE_LOAD_CALLBACK_PLUGINS': 'yes',
            'ANSIBLE_STDOUT_CALLBACK': 'json',
        }

    def bake(self, verbose=True):
        args = [
            '--extra-vars', f'enough_domain_config_directory={self.config_dir}',
        ]
        args.extend(self.inventories)
        if self.vault_password_option():
            args.append(self.vault_password_option())
        logger = logging.getLogger(__name__)
        ansible_env = os.environ.copy()
        ansible_env.update(self.get_env())
        kwargs = dict(
            _truncate_exc=False,
            _env=ansible_env,
        )
        if verbose:
            kwargs.update(
                _tee=True,
                _out=lambda x: logger.info(x.strip()),
                _err=lambda x: logger.info(x.strip()),
            )
        return self.bake_method.bake(*args, **kwargs)

    def vault_password_option(self):
        password_file = f'{self.config_dir}.pass'
        if os.path.exists(password_file):
            return f'--vault-password-file={password_file}'
        else:
            log.debug(f'no decryption because {password_file} does not exist')
            return None

    def ansible_inventory(self):
        args = [
            '--list',
        ] + self.inventories
        if self.vault_password_option():
            args.append(self.vault_password_option())
        i = sh.ansible_inventory(
            *args,
            _cwd=self.share_dir,
        )
        return json.loads(i.stdout.decode('utf-8'))

    def _flat_inventory(self, i, content):
        hosts = content.get('hosts', [])
        for child in content.get('children', []):
            hosts.extend(self._flat_inventory(i, i.get(child, {})))
        return hosts

    def get_groups(self):
        i = self.ansible_inventory()
        del i['_meta']
        groups = {}
        for name, content in i.items():
            groups[name] = self._flat_inventory(i, content)
        return groups

    def get_global_variable(self, variable):
        # the variable we're looking for is not bound to a host but it
        # can only be found by looking into the variables of a
        # host. Since it is global, we just pick one at random.
        value = [v for k, v in self.get_hostvars(variable, 'all[0]').items()
                 if v != 'VARIABLE IS NOT DEFINED!']
        if not value:
            raise KeyError("%r is not defined" % variable)
        return value[0]

    def get_variable(self, variable, host):
        """Render variable for one inventory host

        :param variable: the variable to render. `hostvars[inventory_hostname]`
            is used when the provided ``variable`` is ``None``.
        :type variable: str
        :param host: the inventory host for which the variable is rendered
        :type hosts: str

        :raises: :class:`sh.ErrorReturnCode_1`: when the host is unknown
        :raises: :class:`KeyError`: when the host is known but the variable is
            undefined

        :returns: the deserialized instance of the JSON rendered variable
        """
        return self.get_hostvars(variable, host)[host]

    def get_hostvars(self, variable, *hosts):
        """Render variable for hosts

        :param variable: the variable to render. `hostvars[inventory_hostname]`
            is used when the provided ``variable`` is ``None``.
        :type variable: str
        :param hosts: the list of hosts for which the variable is rendered,
            defaults to all.
        :type hosts: list of inventory hostname, optional

        :raises: :class:`sh.ErrorReturnCode_1`: when an host is unknown
        :raises: :class:`KeyError`: when all host are known but the variable is
            undefined

        :returns: the deserialized instance of the JSON rendered variable
        """
        if not variable:
            variable = "hostvars[inventory_hostname]"

        args = [
            'all',
            '-mdebug',
            f'-avar="{variable}"',
        ]
        if hosts:
            args.extend(('--limit', ','.join(hosts)))
        r = self.bake(verbose=False)(*args)
        out = json.loads(str(r))['plays'][0]['tasks'][0]['hosts']
        values = {k: v.get(variable) for k, v in out.items()}
        for k, v in values.items():
            if v == 'VARIABLE IS NOT DEFINED!':
                raise AnsibleVariableNotFound("%r is not defined" % variable)
        return values


class Playbook(Ansible):

    bake_method = sh.ansible_playbook

    class NoPasswordException(Exception):
        pass

    @staticmethod
    def is_encrypted(p):
        if not os.path.exists(p):
            return False
        c = open(p).read()
        return c.startswith('$ANSIBLE_VAULT')

    @staticmethod
    def encrypted_files(d):
        return [
            f'{d}/infrastructure_key',
            f'{d}/inventory/group_vars/all/clouds.yml',
        ] + glob.glob(f'{d}/certs/*.key')

    def ensure_decrypted(self):
        encrypted = [f for f in self.encrypted_files(self.config_dir)
                     if self.is_encrypted(f)]
        if len(encrypted) == 0:
            return False
        vault_password_option = self.vault_password_option()
        if not vault_password_option:
            raise Playbook.NoPasswordException(
                f'{encrypted} are encrypted but {self.config_dir}.pass does not exist')
        for f in encrypted:
            log.info(f'decrypt {f}')
            sh.ansible_vault.decrypt(
                vault_password_option,
                f,
                _tee=True,
                _out=lambda x: log.info(x.strip()),
                _err=lambda x: log.info(x.strip()),
                _truncate_exc=False,
                _env={
                    'ANSIBLE_NOCOLOR': 'true',
                }
            )
        return True

    @staticmethod
    def roles_path(d):
        r = glob.glob(f'{d}/playbooks/*/roles')
        r.append(f'{d}/playbooks/wazuh/wazuh-ansible/roles/wazuh')
        return ":".join(r)

    def get_env(self):
        ansible_env = super().get_env().copy()
        ansible_env.pop('ANSIBLE_STDOUT_CALLBACK')
        ansible_env.update(ANSIBLE_ROLES_PATH=self.roles_path(self.share_dir))
        return ansible_env

    def run_from_cli(self, **kwargs):
        if not kwargs['args']:
            args = [
                '--private-key', f'{self.config_dir}/infrastructure_key',
                f'{self.config_dir}/enough-playbook.yml'
            ]
        else:
            args = kwargs['args'][1:]
        self.run(*args)

    def run(self, *args):
        self.ensure_decrypted()
        self.bake()(*args)

    def get_role_variable(self, role, variable, host):
        """Return the value of a role default variable in context of host.

        The role default variable can be overriden by an inventory variable.

        :param role: name of the role
        :type role: str
        :param variable: variable name
        :type variable: str
        :param host: inventory hostname
        :type host: str

        :raises: :class:`sh.ErrorReturnCode_1`: when the host is unknown
        :raises: :class:`sh.ErrorReturnCode_2`: when the host is known but the
            variable is undefined

        :returns: the value of the rendered variable.
        """
        return self.get_role_variables(role, variable, host)[host]

    def get_role_variables(self, role, variable, *hosts):
        """Return the values of a role default variable in context of hosts.

        The role default variable can be overriden by an inventory variable.

        :param role: name of the role
        :type role: str
        :param variable: variable name
        :type variable: str
        :param hosts: list of inventory hostnames
        :type hosts: list of str

        :raises: :class:`sh.ErrorReturnCode_1`: when the host is unknown
        :raises: :class:`sh.ErrorReturnCode_2`: when the host is known but the
            variable is undefined

        :returns: a dict where keys are inventory hostnames and values are the
            rendered variable.
        """
        with tempfile.NamedTemporaryFile() as f:
            # the sourrounding "> <" are to prevent conversion to int, list or whatever
            playbook = textwrap.dedent("""
            ---
            - hosts: all
              gather_facts: false
              serial: 1

              roles:
                - role: "{{ rolevar }}"
                  when: false

              tasks:
                - name: print variable
                  debug:
                    msg: ">{{ ansible_play_batch[0] }}:{{ variable }}<"
            """)
            f.write(bytearray(playbook, 'utf-8'))
            f.flush()
            args = [
                '-e', f'rolevar={role}',
                '-e', 'variable={{ ' + variable + ' }}',
                '--limit', ','.join(hosts),
                f.name,
            ]
            r = self.bake(verbose=False)(*args)
            return dict(re.findall(r'.*"msg": ">(.*?):(.*)<"$',
                                   r.stdout.decode('utf-8'),
                                   re.MULTILINE))



from cliff.show import ShowOne

from enough import settings
from enough.common import options
from enough.common import Enough


class Create(ShowOne):
    "Create or update a service"

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument('name')
        parser.add_argument('--playbook', default='enough-playbook.yml')
        parser.add_argument('--host')
        return options.set_options(parser)

    def take_action(self, parsed_args):
        args = vars(self.app.options)
        args.update(vars(parsed_args))
        e = Enough(settings.CONFIG_DIR, settings.SHARE_DIR, **args)
        r = e.service.create_or_update()
        columns = ('name',)
        data = (r['fqdn'],)
        return (columns, data)



testinfra_hosts = ['ansible://bind-host']


def test_spf(host):
    with host.sudo():
        host.run("apt-get install -y dnsutils")
    domain = host.run("hostname -d").stdout.strip()
    cmd = host.run("dig +short TXT " + domain)
    assert 0 == cmd.rc
    assert "v=spf1" in cmd.stdout.strip()



# (c) 2014 Tim Raasveld <info@webtrein.nl>
# https://github.com/timraasveld/ansible-string-split-filter/
# (c) 2014 Maciej Delmanowski <drybjed@gmail.com>
# https://debops.org/


# License: CC0 1.0 Universal
#
# Statement of Purpose
#
# The laws of most jurisdictions throughout the world automatically confer
# exclusive Copyright and Related Rights (defined below) upon the creator and
# subsequent owner(s) (each and all, an "owner") of an original work of
# authorship and/or a database (each, a "Work").
#
# Certain owners wish to permanently relinquish those rights to a Work for the
# purpose of contributing to a commons of creative, cultural and scientific
# works ("Commons") that the public can reliably and without fear of later
# claims of infringement build upon, modify, incorporate in other works, reuse
# and redistribute as freely as possible in any form whatsoever and for any
# purposes, including without limitation commercial purposes. These owners may
# contribute to the Commons to promote the ideal of a free culture and the
# further production of creative, cultural and scientific works, or to gain
# reputation or greater distribution for their Work in part through the use and
# efforts of others.
#
# For these and/or other purposes and motivations, and without any expectation
# of additional consideration or compensation, the person associating CC0 with
# a Work (the "Affirmer"), to the extent that he or she is an owner of
# Copyright and Related Rights in the Work, voluntarily elects to apply CC0 to
# the Work and publicly distribute the Work under its terms, with knowledge of
# his or her Copyright and Related Rights in the Work and the meaning and
# intended legal effect of CC0 on those rights.
#
# 1. Copyright and Related Rights. A Work made available under CC0 may be
# protected by copyright and related or neighboring rights ("Copyright and
# Related Rights"). Copyright and Related Rights include, but are not limited
# to, the following:
#
#   i. the right to reproduce, adapt, distribute, perform, display,
#   communicate, and translate a Work;
#
#   ii. moral rights retained by the original author(s) and/or performer(s);
#
#   iii. publicity and privacy rights pertaining to a person's image or
#   likeness depicted in a Work;
#
#   iv. rights protecting against unfair competition in regards to a Work,
#   subject to the limitations in paragraph 4(a), below;
#
#   v. rights protecting the extraction, dissemination, use and reuse of data
#   in a Work;
#
#   vi. database rights (such as those arising under Directive 96/9/EC of the
#   European Parliament and of the Council of 11 March 1996 on the legal
#   protection of databases, and under any national implementation thereof,
#   including any amended or successor version of such directive); and
#
#   vii. other similar, equivalent or corresponding rights throughout the world
#   based on applicable law or treaty, and any national implementations
#   thereof.
#
# 2. Waiver. To the greatest extent permitted by, but not in contravention of,
# applicable law, Affirmer hereby overtly, fully, permanently, irrevocably and
# unconditionally waives, abandons, and surrenders all of Affirmer's Copyright
# and Related Rights and associated claims and causes of action, whether now
# known or unknown (including existing as well as future claims and causes of
# action), in the Work (i) in all territories worldwide, (ii) for the maximum
# duration provided by applicable law or treaty (including future time
# extensions), (iii) in any current or future medium and for any number of
# copies, and (iv) for any purpose whatsoever, including without limitation
# commercial, advertising or promotional purposes (the "Waiver"). Affirmer
# makes the Waiver for the benefit of each member of the public at large and to
# the detriment of Affirmer's heirs and successors, fully intending that such
# Waiver shall not be subject to revocation, rescission, cancellation,
# termination, or any other legal or equitable action to disrupt the quiet
# enjoyment of the Work by the public as contemplated by Affirmer's express
# Statement of Purpose.
#
# 3. Public License Fallback. Should any part of the Waiver for any reason be
# judged legally invalid or ineffective under applicable law, then the Waiver
# shall be preserved to the maximum extent permitted taking into account
# Affirmer's express Statement of Purpose. In addition, to the extent the
# Waiver is so judged Affirmer hereby grants to each affected person
# a royalty-free, non transferable, non sublicensable, non exclusive,
# irrevocable and unconditional license to exercise Affirmer's Copyright and
# Related Rights in the Work (i) in all territories worldwide, (ii) for the
# maximum duration provided by applicable law or treaty (including future time
# extensions), (iii) in any current or future medium and for any number of
# copies, and (iv) for any purpose whatsoever, including without limitation
# commercial, advertising or promotional purposes (the "License"). The License
# shall be deemed effective as of the date CC0 was applied by Affirmer to the
# Work. Should any part of the License for any reason be judged legally invalid
# or ineffective under applicable law, such partial invalidity or
# ineffectiveness shall not invalidate the remainder of the License, and in
# such case Affirmer hereby affirms that he or she will not (i) exercise any of
# his or her remaining Copyright and Related Rights in the Work or (ii) assert
# any associated claims and causes of action with respect to the Work, in
# either case contrary to Affirmer's express Statement of Purpose.
#
# 4. Limitations and Disclaimers.
#
#   a. No trademark or patent rights held by Affirmer are waived, abandoned,
#   surrendered, licensed or otherwise affected by this document.
#
#   b. Affirmer offers the Work as-is and makes no representations or
#   warranties of any kind concerning the Work, express, implied, statutory or
#   otherwise, including without limitation warranties of title,
#   merchantability, fitness for a particular purpose, non infringement, or the
#   absence of latent or other defects, accuracy, or the present or absence of
#   errors, whether or not discoverable, all to the greatest extent permissible
#   under applicable law.
#
#   c. Affirmer disclaims responsibility for clearing rights of other persons
#   that may apply to the Work or any use thereof, including without limitation
#   any person's Copyright and Related Rights in the Work. Further, Affirmer
#   disclaims responsibility for obtaining any necessary consents, permissions
#   or other rights required for any use of the Work.
#
#   d. Affirmer understands and acknowledges that Creative Commons is not a
#   party to this document and has no duty or obligation with respect to this
#   CC0 or use of the Work.
#
# For more information, please see
# <http://creativecommons.org/publicdomain/zero/1.0/>


import re


def split_string(string, seperator=None, maxsplit=-1):
    try:
        return string.split(seperator, maxsplit)
    except Exception:
        return list(string)


def split_regex(string, seperator_pattern):
    try:
        return re.split(seperator_pattern, string)
    except Exception:
        return list(string)


class FilterModule(object):
    ''' A filter to split a string into a list. '''
    def filters(self):
        return {
            'split': split_string,
            'split_regex': split_regex,
        }



testinfra_hosts = ['ansible://bind-client-host']


def test_caa(host):
    domain = host.run("hostname -d").stdout.strip()
    cmd = host.run("dig +short CAA " + domain)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc
    assert '"letsencrypt.org"' in cmd.stdout.strip()



import argparse

from enough.common import options


def test_set_options():
    parser = argparse.ArgumentParser()
    assert options.set_options(parser) == parser
    args = parser.parse_args([])
    driver = 'DRIVER'
    args = parser.parse_args(['--driver', driver])
    assert args.driver == driver



import logging
import sys
import json
import random
import string
import argparse
import os

# Set framework path
sys.path.append("/var/ossec/framework")

try:
    from wazuh.security import (
        create_user,
        get_users,
        get_roles,
        set_user_role,
        update_user,
    )
except Exception as e:
    logging.error("No module 'wazuh' found.")
    sys.exit(1)


def db_users():
    users_result = get_users()
    return {user["username"]: user["id"] for user in users_result.affected_items}


def db_roles():
    roles_result = get_roles()
    return {role["name"]: role["id"] for role in roles_result.affected_items}


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='add_user script')
    parser.add_argument('--username', action="store", dest="username")
    parser.add_argument('--password', action="store", dest="password")
    results = parser.parse_args()

    username = results.username
    password = results.password

    initial_users = db_users()
    if username not in initial_users:
        # create a new user
        create_user(username=username, password=password)
        users = db_users()
        uid = users[username]
        roles = db_roles()
        rid = roles["administrator"]
        set_user_role(
            user_id=[
                str(uid),
            ],
            role_ids=[
                str(rid),
            ],
        )
    else:
        # modify an existing user ("wazuh" or "wazuh-wui")
        uid = initial_users[username]
        update_user(
            user_id=[
                str(uid),
            ],
            password=password,
        )
    # set a random password for all other users
    for name, id in initial_users.items():
        if name != username:
            specials = "@$!%*?&-_"
            random_pass = "".join(
                [
                    random.choice(string.ascii_uppercase),
                    random.choice(string.ascii_lowercase),
                    random.choice(string.digits),
                    random.choice(specials),
                ] +
                random.choices(
                    string.ascii_uppercase
                    + string.ascii_lowercase
                    + string.digits
                    + specials,
                    k=14,
                )
            )
            update_user(
                user_id=[
                    str(id),
                ],
                password=random_pass,
            )



import pytest

from enough import settings
from enough.common import ansible_utils, dotenough


def pytest_configure(config):
    config.addinivalue_line("markers", "openstack_integration: mark tests "
                            "which require OpenStack credentials")


@pytest.fixture(scope='session')
def openstack_variables(request):
    ansible = ansible_utils.Ansible(settings.CONFIG_DIR, settings.SHARE_DIR)
    os_variables = ('openstack_flavor', 'openstack_image', 'openstack_network',
                    'network_primary_interface', 'network_secondary_interface',
                    '_provider')
    os_variables = "{%s}" % ', '.join(f'"{x}": {x}' for x in os_variables)
    # Any host could be used
    hostvars = ansible.get_variable(os_variables, 'bind-host')
    hostvars['openstack_provider'] = hostvars.pop('_provider')
    return hostvars


@pytest.fixture
def dot_openstack(tmp_config_dir, request):
    domain = settings.CONFIG_DIR.split('/')[-1]
    dot_openstack = dotenough.DotEnoughOpenStack(settings.CONFIG_DIR, domain)
    dot_openstack.ensure()
    yield dot_openstack



# -*- coding: utf-8 -*-
#
# DebOps documentation build configuration file
# Copyright (C) 2014-2018 DebOps Project https://debops.org/
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os

import sphinx.environment
from docutils.utils import get_source_line

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('_lib'))

import yaml2rst  # noqa
import edit_url  # noqa

rst_ansible_roles = 'ansible/roles/'
yml_ansible_roles = '../ansible/roles/'

# Convert Ansible role defaults files written in YAML to documentation written
# in reStructuredText
for element in os.listdir(rst_ansible_roles):
    if os.path.isdir(yml_ansible_roles + element + '/defaults'):
        for path, subdirs, files in os.walk(yml_ansible_roles +
                                            element + '/defaults'):
            for filename in files:
                if not filename.startswith('.'):
                    defaults_file = os.path.join(path, filename)
                    defaults_dir = os.path.dirname(defaults_file).lstrip('../')

                    if not os.path.isdir(defaults_dir):
                        try:
                            os.makedirs(defaults_dir)
                        except OSError:
                            print("Creation of the directory %s failed"
                                  % defaults_dir)

                    yaml2rst.convert_file(
                        defaults_file,
                        (os.path.splitext(defaults_file)[0]
                            + '.rst').lstrip('../'),
                        strip_regex=r'\s*(:?\[{3}|\]{3})\d?$',
                        yaml_strip_regex=r'^\s{66,67}#\s\]{3}\d?$',
                    )


# Ignore warnings about non-local images
def _warn_node(self, msg, node, **kwargs):
    if not msg.startswith('nonlocal image URI found:'):
        self._warnfunc(msg, '%s:%s' % get_source_line(node))


sphinx.environment.BuildEnvironment.warn_node = _warn_node

git_commit_id = os.popen('git rev-parse --short HEAD').read().strip()

html_context = {
    'display_github': True,  # # Add 'Edit on Github' link
    'github_user': 'debops',
    'github_repo': 'debops',
    'github_version': 'master',
    'conf_py_path': '/docs/',
    'commit': git_commit_id,
    'source_file_to_url_map': edit_url.get_source_file_to_url_map(
        start_dir=os.path.dirname(__file__),
        skip_patterns=[]
    )
}


# Add custom .css and .js files from '_static/' directory in a way compatible
# with ReadTheDocs.org deployment
# Source: https://github.com/rtfd/sphinx_rtd_theme/issues/117
def setup(app):
    # app.add_javascript("custom.js")
    app.add_stylesheet("theme_overrides.css")


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.ifconfig',
    'sphinx.ext.graphviz',
    'autolink'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'DebOps'
author = u'Maciej Delmanowski, Nick Janetakis, Robin Schneider'
copyright = u'2014-2019, {}'.format(author)

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The full version, including alpha/beta/rc tags.
release = os.popen('git describe').read().strip()
# The short X.Y version.
version = release

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build', 'includes']

# The reST default role (used for this markup: `text`) to use for all
# documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# The default language to highlight source code in. The default is 'python'.
# The value should be a valid Pygments lexer name, see Showing code examples
# for more details.
highlight_language = 'YAML'

# TODO: Change later to this when it can handle:
# enabled: '{{ True
#              if (owncloud_database_name != owncloud_database_user)
#              else False }}'
#  highlight_language = 'YAML+Jinja'

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'default'

# Check if build is performed on ReadTheDocs.org infrastructure
on_rtd = os.environ.get('READTHEDOCS', None) == 'True'

if not on_rtd:  # only import and set the theme if we're building docs locally
    try:
        import sphinx_rtd_theme
        html_theme = 'sphinx_rtd_theme'
        html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
        html_theme_options = {
            'collapse_navigation': True,
            'logo_only': False
        }
    except Exception:
        pass

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = '_static/images/debops-small.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = '_static/favicon.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
# html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = False

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'DebOpsdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    'papersize': 'a4paper',

    # The font size ('10pt', '11pt' or '12pt').
    'pointsize': '11pt',

    # Additional stuff for the LaTeX preamble.
    'preamble': r'''
        \usepackage{charter}
        \usepackage[defaultsans]{lato}
        \usepackage{inconsolata}
    ''',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', 'DebOps.tex', u'DebOps Documentation',
   u'DebOps Developers', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
latex_logo = '_static/images/debops-small.png'

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'debops', u'DebOps Documentation',
     [author], 1)
]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', 'DebOps', u'DebOps Documentation',
   author, 'DebOps', 'Your Debian-based data center in a box',
   'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
# texinfo_no_detailmenu = False


# -- Options for Epub output ----------------------------------------------

# Bibliographic Dublin Core info.
epub_title = u'DebOps'
epub_author = author
epub_publisher = author
epub_copyright = u'2014-2019, Maciej Delmanowski'

# The basename for the epub file. It defaults to the project name.
# epub_basename = u'DebOps'

# The HTML theme for the epub output. Since the default themes are not
# optimized for small screen space, using the same theme for HTML and epub
# output is usually not wise. This defaults to 'epub', a theme designed to save
# visual space.
# epub_theme = 'epub'

# The language of the text. It defaults to the language option
# or en if the language is not set.
# epub_language = ''

# The scheme of the identifier. Typical schemes are ISBN or URL.
# epub_scheme = ''

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
# epub_identifier = ''

# A unique identification for the text.
# epub_uid = ''

# A tuple containing the cover image and cover page html template filenames.
# epub_cover = ()

# A sequence of (type, uri, title) tuples for the guide element of content.opf.
# epub_guide = ()

# HTML files that should be inserted before the pages created by sphinx.
# The format is a list of tuples containing the path and title.
# epub_pre_files = []

# HTML files shat should be inserted after the pages created by sphinx.
# The format is a list of tuples containing the path and title.
# epub_post_files = []

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']

# The depth of the table of contents in toc.ncx.
# epub_tocdepth = 3

# Allow duplicate toc entries.
# epub_tocdup = True

# Choose between 'default' and 'includehidden'.
# epub_tocscope = 'default'

# Fix unsupported image types using the PIL.
# epub_fix_images = False

# Scale large images.
# epub_max_image_width = 0

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# epub_show_urls = 'inline'

# If false, no index is generated.
# epub_use_index = True



testinfra_hosts = ['ansible://ownca-host']


def test_can_curl_locally(host):
    with host.sudo():
        host.run("apt-get install -y curl")
    assert host.run("curl -m 5 -I https://ownca-host.$(hostname -d)").rc == 0
    assert host.run("grep 'SOMETHING TESTS CAN GREP' /etc/nginx/sites-enabled/*.conf").rc == 0


def test_letsencryptstaging_fails_because_no_ownca_installed(host):
    letsencryptstaging_host = host.get_host('ansible://letsencryptstaging-host',
                                            ansible_inventory=host.backend.ansible_inventory)
    with letsencryptstaging_host.sudo():
        letsencryptstaging_host.run("apt-get install -y curl")
    assert letsencryptstaging_host.run("curl -m 5 -I -k https://ownca-host.$(hostname -d)").rc == 0
    assert letsencryptstaging_host.run("curl -m 5 -I https://ownca-host.$(hostname -d)").rc == 60


def test_client_succeeds_because_ownca_installed(host):
    client_host = host.get_host('ansible://client-host',
                                ansible_inventory=host.backend.ansible_inventory)
    with client_host.sudo():
        client_host.run("apt-get install -y curl")
    assert client_host.run("curl -m 5 -I https://ownca-host.$(hostname -d)").rc == 0



# Copyright (C) 2017 Maciej Delmanowski <drybjed@gmail.com>
# Copyright (C) 2017 DebOps https://debops.org/

# This file is part of DebOps.
#
# DebOps is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DebOps is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DebOps. If not, see <https://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
from operator import itemgetter

try:
    unicode = unicode
except NameError:
    # py3
    unicode = str

__metaclass__ = type


def _mangle_recipients(name, *args):
    """Modify the recipient address if it's the same as alias
    to prevent mail forwarding loops
    Ref: https://serverfault.com/questions/471604/
    """

    input_args = []

    # Flatten the input list
    for sublist in list(args):
        for item in sublist:
            input_args.append(item)

    if name in input_args:
        return [w.replace(name, '\\' + name) for w in input_args]
    else:
        return input_args


def _update_recipients(current_data, new_data, *args, **kwargs):
    """Replace the current list of recipients with a new one"""

    for selector in args:
        if selector in new_data:
            new_recipients = ([new_data.get(selector)]
                              if isinstance(new_data.get(selector),
                                            (str, unicode))
                              else new_data.get(selector))
            current_data.update({'recipients':
                                _mangle_recipients(current_data.get('name'),
                                                   new_recipients)})


def _add_recipients(current_data, new_data, *args, **kwargs):
    """Add mail recipients to an existing list of recipients"""

    for selector in args:
        if selector in new_data:
            current_recipients = current_data.get('recipients', [])
            current_recipients.extend([new_data.get(selector)]
                                      if isinstance(new_data.get(selector),
                                                    (str, unicode))
                                      else new_data.get(selector))
            current_data.update({'recipients':
                                _mangle_recipients(current_data.get('name'),
                                                   current_recipients)})


def _del_recipients(current_data, new_data, *args, **kwargs):
    """Remove mail recipients from an existing list"""

    for selector in args:
        if selector in new_data:
            deleted_recipients = ([new_data.get(selector)]
                                  if isinstance(new_data.get(selector),
                                                (str, unicode))
                                  else new_data.get(selector))
            current_recipients = current_data.get('recipients', [])
            new_recipients = [x for x in current_recipients
                              if x not in deleted_recipients]
            current_data.update({'recipients': new_recipients})


def etc_aliases__parse_recipients(*args, **kwargs):
    """Return a parsed list of mail aliases and recipients"""

    input_args = []
    parsed_aliases = {}

    # Flatten the input list
    for sublist in list(args):
        for item in sublist:
            input_args.append(item)

    for element in input_args:
        if isinstance(element, dict):
            if (any(x in element for x in ['name', 'alias']) and
                    element.get('state', 'present') != 'ignore'):

                alias_name = element.get('alias', element.get('name'))
                current_alias = (parsed_aliases[alias_name].copy()
                                 if alias_name in parsed_aliases
                                 else {})

                current_alias.update({
                    'name': alias_name,  # in case of a new entry
                    'state': element.get('state',
                                         current_alias.get('state',
                                                           'present')),
                    'weight': int(element.get('weight',
                                  current_alias.get('weight', 0))),
                    'section': element.get('section',
                                           current_alias.get('section',
                                                             'unknown'))
                })

                _update_recipients(current_alias, element, 'dest', 'to')

                _add_recipients(current_alias, element,
                                'add_dest', 'add_to', 'cc', 'bcc')

                _del_recipients(current_alias, element,
                                'del_dest', 'del_to')

                if 'real_name' in element or 'real_alias' in element:
                    current_alias['real_name'] = (
                        element.get('real_name',
                                    element.get('real_alias')))

                if 'comment' in element:
                    current_alias['comment'] = element.get('comment')

                if not current_alias.get('recipients', ''):
                    current_alias['state'] = 'comment'

                parsed_aliases.update({alias_name: current_alias})

            # These parameters are special and should not be interpreted
            # directly as mail aliases.
            elif not all(x in ['name', 'alias', 'state', 'comment',
                               'section', 'weight', 'dest', 'to',
                               'add_dest', 'add_to', 'cc', 'bcc',
                               'del_dest', 'del_to']
                         for x in element):
                for key, value in element.items():
                    current_alias = parsed_aliases.get(key, {}).copy()
                    current_alias.update({
                        'name': key,
                        'recipients': (_mangle_recipients(
                                       current_alias.get('name'), [value]
                                       if isinstance(value, (str, unicode))
                                       else value)),
                        'state': 'present',
                        'weight': int(element.get('weight',
                                      current_alias.get('weight', 0))),
                        'section': current_alias.get('section', 'unknown')
                    })

                    current_alias.update({
                        'recipients': (_mangle_recipients(
                                       current_alias.get('name'),
                                       ([value]
                                        if isinstance(value, (str, unicode))
                                        else value)))
                    })

                    if not current_alias.get('recipients', ''):
                        current_alias['state'] = 'comment'

                    parsed_aliases[key] = current_alias

    # Expand the dictionary of aliases into a list,
    # and return sorted by weight.
    return sorted(parsed_aliases.values(), key=itemgetter('weight', 'name'))


class FilterModule(object):
    """Register custom filter plugins in Ansible"""

    def filters(self):
        return {'etc_aliases__parse_recipients': etc_aliases__parse_recipients}



import pytest
from django.urls import reverse
from bs4 import BeautifulSoup


@pytest.mark.django_db
def test_view(client, django_user_model):
    django_user_model.objects.create_user(
        username='test', email='test@...', password='top_secret')
    client.login(username='test', password='top_secret')
    response = client.get(reverse('user_home'))
    soup = BeautifulSoup(response.content, 'html.parser')
    token = soup.select('#token')
    assert len(token) == 1
    assert len(token[0].contents[0]) == 40
    assert response.status_code == 200



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 DebOps project https://debops.org/

from docutils import nodes


def setup(app):
    app.add_role('man', autolink('https://manpages.debian.org/%s'))


def autolink(pattern):
    def role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        url = pattern % (text,)
        node = nodes.reference(rawtext, text, refuri=url, **options)
        return [node], []
    return role



from pprint import pprint
import testinfra
import requests
import yaml
from enough.common import ansible_utils
from enough.common import retry

testinfra_hosts = ['ansible://wazuh-host']


class Wazuh(object):

    def __init__(self, config):
        inventory = config.getoption("--ansible-inventory")
        url = self.get_address(inventory)
        p = ansible_utils.Ansible('.', '.', ['playbooks/wazuh/inventory'])
        username = p.get_variable('wazuh_api_username', 'wazuh-host')
        password = p.get_variable('wazuh_api_password', 'wazuh-host')
        self.url = f'https://{url}:55000'
        self.s = requests.session()
        self.s.verify = None
        self.s.headers = {
            'Accept': 'application/json',
        }
        r = self.s.get(f'{self.url}/security/user/authenticate',
                       auth=(username, password))
        print(r.text)
        r.raise_for_status()
        self.s.headers['Authorization'] = f"Bearer {r.json()['data']['token']}"

    def get_info(self):
        r = self.s.get(self.url, params={'pretty': 'true'})
        print(r.text)
        r.raise_for_status()
        return r.json()

    def get_agents(self):
        r = self.s.get(f'{self.url}/agents')
        print(r.text)
        r.raise_for_status()
        return r.json()

    def get_agent_id(self, host):
        agents = self.get_agents()['data']['affected_items']
        for agent in agents:
            if agent['name'].startswith(host):
                return agent['id']
        raise Exception(f'{host} not found in {agents}')

    def get_address(self, inventory):
        vars_dir = f'{inventory}/group_vars/all'
        return 'wazuh.' + yaml.safe_load(
            open(vars_dir + '/domain.yml'))['domain']

    def get_check_times(self, agent):
        r = self.s.get(f'{self.url}/syscheck/{agent}/last_scan')
        print(r.text)
        r.raise_for_status()
        d = r.json()['data']['affected_items'][0]
        return (d['start'], d['end'])

    @retry.retry(AssertionError, tries=8)
    def wait_for_checks(self, agent):
        # start time > end time means the check is ongoing
        (start, end) = self.get_check_times(agent)
        assert start < end

    def get_syscheck_end(self, agent):
        return self.get_check_times(agent)[1]

    def run_syscheck(self, host):
        agent = self.get_agent_id(host)
        last = self.get_syscheck_end(agent)
        r = self.s.put(self.url + '/syscheck', params={'agents_list': agent})
        r.raise_for_status()
        d = r.json()
        pprint(d)
        assert d['error'] == 0

        @retry.retry(AssertionError, tries=8)
        def wait_for_syscheck():
            current_last = self.get_syscheck_end(agent)
            assert current_last is not None, f'syscheck in progress'
            assert current_last > last, f'{current_last} > {last}'
        wait_for_syscheck()

    @retry.retry(AssertionError, tries=8)
    def get_syscheck_md5(self, host, path):
        agent = self.get_agent_id(host)
        r = self.s.get(f'{self.url}/syscheck/{agent}', params={'file': path})
        print(r.text)
        r.raise_for_status()
        r = r.json()
        assert len(r['data']['affected_items']) > 0, f'{r} has no information yet about {path}'
        d = r['data']['affected_items'][0]
        assert d['file'] == path
        return d['md5']


def test_wazuh_api(host, pytestconfig):
    w = Wazuh(pytestconfig)
    info = w.get_info()
    assert info['error'] == 0
    assert info['data']['title'] == "Wazuh API REST"


def test_wazuh_syscheck(host, pytestconfig):
    try:
        w = Wazuh(pytestconfig)
        w.run_syscheck('postfix-host')
        good_md5 = w.get_syscheck_md5('postfix-host', '/etc/screenrc')
        with host.sudo():
            host.run("""
            postsuper -d ALL
            sed -i -e '/email_alert_level/s/>12</>6</' /var/ossec/etc/ossec.conf
            systemctl restart wazuh-manager
            """)
        # postfix_host is a wazuh agent
        postfix_host = testinfra.host.Host.get_host(
            'ansible://postfix-host',
            ansible_inventory=host.backend.ansible_inventory)

        #
        # modify /etc/screenrc and verify wazuh sees it
        #
        with postfix_host.sudo():
            postfix_host.run("date +%s >> /etc/screenrc")
        w.run_syscheck('postfix-host')
        bad_md5 = w.get_syscheck_md5('postfix-host', '/etc/screenrc')
        assert good_md5 != bad_md5

        #
        # verify a notification is sent regarding this change
        #
        @retry.retry(AssertionError, tries=5)
        def wait_for_mail():
            with host.sudo():
                cmd = host.run("""
                ls /var/spool/postfix/hold
                grep -qw /etc/screenrc /var/spool/postfix/hold/*
                """)
            print(cmd.stdout)
            assert cmd.rc == 0, f'{cmd.stdout} {cmd.stderr}'
        wait_for_mail()

    finally:
        with host.sudo():
            host.run("""
            sed -i -e '/email_alert_level/s/>.*</>12</' /var/ossec/etc/ossec.conf
            systemctl restart wazuh-manager
            """)



# -*- coding: utf-8 -*-

# Copyright (C) 2015 Hartmut Goebel <h.goebel@crazy-compilers.com>
# Part of the DebOps - https://debops.org/

# This program is free software; you can redistribute
# it and/or modify it under the terms of the
# GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General
# Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# An on-line copy of the GNU General Public License can
# be downloaded from the FSF web page at:
# https://www.gnu.org/copyleft/gpl.html

import os
import sys
from io import StringIO
try:
    import configparser
except ImportError:
    import ConfigParser as configparser

__all__ = ['DEBOPS_CONFIG', 'read_config']

__author__ = "Hartmut Goebel <h.goebel@crazy-compilers.com>"
__copyright__ = "Copyright 2015 by Hartmut Goebel "
"<h.goebel@crazy-compilers.com>"
__licence__ = "GNU General Public License version 3 (GPL v3) or later"


DEBOPS_CONFIG = ".debops.cfg"

DEFAULTS = u"""
[paths]
data-home: $XDG_DATA_HOME/debops

# Default monorepo directory
monorepo-path: %(data-home)s/debops

# Default installation directory
install-path: %(data-home)s/debops

# Locations where DebOps playbooks might be found
# This MUST be a multi-line string to make ConfigParser work
playbooks-paths: %(monorepo-path)s/ansible/playbooks

[ansible defaults]
ansible_managed = This file is managed remotely, all changes will be lost
"""

if sys.platform.startswith('win'):
    DEFAULTS = DEFAULTS.replace('$XDG_DATA_HOME',
                                os.getenv('APPDATA') or '~\\Application Data')
elif sys.platform == 'darwin':  # Mac OS X
    DEFAULTS = DEFAULTS.replace('$XDG_DATA_HOME',
                                '~/Library/Application Support')


def _set_xdg_defaults():
    """
    Set default values for XDG variables according to XDG specification
    http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
    """
    for name, default in (
            ('XDG_CONFIG_HOME', '~/.config'),
            ('XDG_CONFIG_DIRS', '/etc/xdg'),
            ('XDG_DATA_HOME', '~/.local/share')):
        if not os.environ.get(name):
            os.environ[name] = default


def _get_config_filenames():
    if sys.platform.startswith('win'):
        configdirs = [os.getenv('APPDATA')
                      or os.path.expanduser('~\\Application Data')]
    elif sys.platform == 'darwin':  # Mac OS X
        configdirs = [os.path.expanduser('~/Library/Application Support'),
                      '/etc']
        configdirs.reverse()
    else:
        _set_xdg_defaults()
        configdirs = ([os.getenv('XDG_CONFIG_HOME')] +
                      os.getenv('XDG_CONFIG_DIRS').split(':') +
                      ['/etc'])
        configdirs = [os.path.expanduser(d) for d in configdirs]
        configdirs.reverse()
    return [os.path.join(d, 'debops.cfg') for d in configdirs]


_configfiles = _get_config_filenames()


def _expandpath(path):
    return os.path.expanduser(os.path.expandvars(path.strip()))


def read_config(project_root):
    if project_root is None:
        configfiles = _configfiles
    else:
        configfiles = _configfiles + [os.path.join(project_root,
                                                   DEBOPS_CONFIG)]
    cfgparser = configparser.SafeConfigParser()
    cfgparser.readfp(StringIO(DEFAULTS))
    try:
        cfgparser.read(configfiles)
    except configparser.Error as e:
        raise SystemExit('Error in %s: %s' % (DEBOPS_CONFIG, str(e)))
    cfg = dict((sect, dict(cfgparser.items(sect)))
               for sect in cfgparser.sections())
    # expand vars and home-directory
    for name in ('data-home', 'monorepo-path', 'install-path'):
        cfg['paths'][name] = _expandpath(cfg['paths'][name])
    cfg['paths']['playbooks-paths'] = [
        _expandpath(p)
        for p in cfg['paths']['playbooks-paths'].splitlines()
        if p.strip()]
    return cfg



from cliff.show import ShowOne

from django.conf import settings

from enough.common import openstack


class CreateTestSubdomain(ShowOne):
    "Create and delegate a test subdomain, if possible"

    def take_action(self, parsed_args):
        h = openstack.Heat(settings.CONFIG_DIR)
        r = h.create_test_subdomain(self.app.options.domain)
        columns = ('name',)
        data = (r,)
        return (columns, data)



import os
import sh
import shutil
import textwrap
import yaml
from enough.common import ansible_utils


class Hosts(object):

    def __init__(self, config_dir):
        self.d = f'{config_dir}/inventory'
        self.f = f'{self.d}/hosts.yml'
        self.load()

    def load(self):
        if os.path.exists(self.f):
            self.hosts = yaml.safe_load(open(self.f).read())['all']['hosts']
        else:
            self.hosts = {}
        return self

    def get_ip(self, host):
        return self.hosts.get(host, {}).get('ansible_host')

    def get_port(self, host):
        return self.hosts.get(host, {}).get('ansible_port', '22')

    def save(self):
        if not os.path.exists(self.d):
            os.makedirs(self.d)
        content = yaml.dump(
            {
                'all': {
                    'hosts': self.hosts,
                },
            }
        )
        open(self.f, 'w').write(content)

    def missings(self, names):
        return [name for name in names if name not in self.hosts]

    def ensure(self, name):
        if name not in self.hosts:
            self.hosts[name] = {}
            self.save()
            return True
        else:
            return False

    def create_or_update(self, name, ipv4, port):
        if self.get_ip(name) != ipv4:
            self.hosts[name] = {'ansible_host': ipv4, 'ansible_port': port}
            self.save()
            return True
        else:
            return False

    def delete(self, name):
        if name in self.hosts:
            del self.hosts[name]
        self.save()


class DotEnough(object):

    def __init__(self, config_dir, domain):
        self.domain = domain
        self.config_dir = config_dir
        d = f'{self.config_dir}/inventory/group_vars/all'
        if not os.path.exists(d):
            os.makedirs(d)

    def ensure(self):
        self.ensure_ssh_key()

    def ensure_ssh_key(self):
        path = f'{self.config_dir}/infrastructure_key'
        if not os.path.exists(path):
            sh.ssh_keygen('-f', path, '-N', '', '-b', '4096', '-t', 'rsa')
        return path

    def public_key(self):
        return f'{self.ensure_ssh_key()}.pub'

    def private_key(self):
        return self.ensure_ssh_key()

    def populate_config(self, certificate_authority):
        d = f'{self.config_dir}/inventory/group_vars/all'
        if not os.path.exists(d):
            os.makedirs(d)

        if not os.path.exists(f'{d}/private-key.yml'):
            open(f'{d}/private-key.yml', 'w').write(textwrap.dedent(f"""\
            ---
            ansible_ssh_private_key_file: {self.config_dir}/infrastructure_key
            """))

        if not os.path.exists(f'{d}/domain.yml'):
            open(f'{d}/domain.yml', 'w').write(textwrap.dedent(f"""\
            ---
            domain: {self.domain}
            production_domain: {self.domain}
            """))

        if not os.path.exists(f'{d}/certificate.yml'):
            self.set_certificate(certificate_authority)

    @staticmethod
    def service2group(service):
        return f'{service}-service-group'

    def service_add_to_group(self, service, host):
        s = f'{self.config_dir}/inventory/services.yml'
        if os.path.exists(s):
            services = yaml.safe_load(open(s).read())
        else:
            services = {}
        group = self.service2group(service)
        if group not in services:
            services[group] = {'hosts': {}}
        if host not in services[group]['hosts']:
            services[group]['hosts'][host] = None
            open(s, 'w').write(yaml.dump(services, indent=4))
        return services

    def set_certificate(self, certificate_authority):
        d = f'{self.config_dir}/inventory/group_vars/all'
        open(f'{d}/certificate.yml', 'w').write(textwrap.dedent(f"""\
        ---
        certificate_authority: {certificate_authority}
        """))


class DotEnoughOpenStackUnknownProvider(Exception):
    pass


class DotEnoughOpenStack(DotEnough):

    def __init__(self, config_dir, domain):
        super().__init__(config_dir, domain)
        self.clouds_file = f'{self.config_dir}/inventory/group_vars/all/clouds.yml'

    def set_clouds_file(self, clouds_file):
        shutil.copy(clouds_file, self.clouds_file)

    def ensure(self):
        super().ensure()
        self.populate_config('letsencrypt')

    def populate_provider(self, ansible, cloud):
        try:
            return ansible.get_global_variable('openstack_provider')
        except ansible_utils.AnsibleVariableNotFound:
            pass

        try:
            clouds = ansible.get_global_variable('clouds')
        except ansible_utils.AnsibleVariableNotFound:
            return None

        url = clouds[cloud]['auth']['auth_url']
        string2provider = (('ovh.net', 'ovh'),
                           ('fuga.cloud', 'fuga'))
        provider = None
        for (string, found) in string2provider:
            if string in url:
                provider = found
                break
        if provider is None:
            raise DotEnoughOpenStackUnknownProvider(
                f'url {url} does not match any known providers {string2provider}')

        f = f'{self.config_dir}/inventory/group_vars/all/openstack-provider.yml'
        open(f, 'w').write(textwrap.dedent(f"""\
        ---
        openstack_provider: {provider}
        """))
        return f


class DotEnoughDocker(DotEnough):

    def ensure(self):
        super().ensure()
        self.populate_config('ownca')



import os
import pytest
import testinfra.utils.ansible_runner

testinfra_hosts = testinfra.utils.ansible_runner.AnsibleRunner(
    os.environ['MOLECULE_INVENTORY_FILE']).get_hosts('all')


def get_wazuh_version():
    """This return the version of Wazuh."""
    return "4.0.1"


def test_wazuh_packages_are_installed(host):
    """Test the main packages are installed."""
    manager = host.package("wazuh-manager")
    assert manager.is_installed
    assert manager.version.startswith(get_wazuh_version())


def test_wazuh_services_are_running(host):
    """Test the services are enabled and running.

    When assert commands are commented, this means that the service command has
    a wrong exit code: https://github.com/wazuh/wazuh-ansible/issues/107
    """
    # This currently doesn't work with out current Docker base images
    # manager = host.service("wazuh-manager")
    # api = host.service("wazuh-api")
    # assert manager.is_running
    # assert api.is_running
    output = host.check_output(
        'ps aux | grep ossec | tr -s " " | cut -d" " -f11'
        )
    assert 'ossec-authd' in output
    assert 'wazuh-modulesd' in output
    assert 'wazuh-db' in output
    assert 'ossec-execd' in output
    assert 'ossec-monitord' in output
    assert 'ossec-remoted' in output
    assert 'ossec-logcollector' in output
    assert 'ossec-analysisd' in output
    assert 'ossec-syscheckd' in output


@pytest.mark.parametrize("wazuh_file, wazuh_owner, wazuh_group, wazuh_mode", [
    ("/var/ossec/etc/sslmanager.cert", "root", "root", 0o640),
    ("/var/ossec/etc/sslmanager.key", "root", "root", 0o640),
    ("/var/ossec/etc/rules/local_rules.xml", "ossec", "ossec", 0o640),
    ("/var/ossec/etc/lists/audit-keys", "ossec", "ossec", 0o660),
])
def test_wazuh_files(host, wazuh_file, wazuh_owner, wazuh_group, wazuh_mode):
    """Test Wazuh related files exist and have proper owners and mode."""
    wazuh_file_host = host.file(wazuh_file)
    assert wazuh_file_host.user == wazuh_owner
    assert wazuh_file_host.group == wazuh_group
    assert wazuh_file_host.mode == wazuh_mode


def test_filebeat_is_installed(host):
    """Test the elasticsearch package is installed."""
    filebeat = host.package("filebeat")
    assert filebeat.is_installed
    assert filebeat.version.startswith('7.9.3')



from nose.tools import assert_equals

from debops.api import DebOpsAPI


def test__get_maintainer_from_changelog():
    debops_api = DebOpsAPI()
    expected = {
        'The current role maintainer_ is drybjed_.': ['drybjed'],
        'The current role maintainers_ are drybjed_ and ypid_.': (
            ['drybjed', 'ypid']),
        'The current role maintainers_ are drybjed_, foo_ and ypid_.': (
            ['drybjed', 'foo', 'ypid']),
        'The current role maintainer is drybjed_.': ['drybjed'],
        'The current role maintainer is drybjed.': ['drybjed'],
        'The current role maintainer is drybjed': ['drybjed'],
        'The current maintainer is drybjed': ['drybjed'],
        'The maintainer is drybjed': ['drybjed'],
        '.. include:: includes/all.rst': None,
    }
    for line, expected_result in expected.items():
        assert_equals(
            expected_result,
            debops_api._get_maintainers_from_line(line)
        )



testinfra_hosts = ['ansible://website-host']


def test_nginx(host):
    with host.sudo():
        host.run("apt-get install -y curl")
    assert host.run("curl -m 5 -I http://website-host.$(hostname -d)").rc == 0
    assert host.run("grep 'SOMETHING TESTS CAN GREP' /etc/nginx/sites-enabled/*.conf").rc == 0



def update(host):
    with host.sudo():
        cmd = host.run("""
        set -xe
        flock /tmp/update-website \
              bash -x /srv/update-website.sh \
              >> /var/log/update-website.log 2>&1
        grep --quiet -i enough /usr/share/nginx/html/index.html
        """)
        print(cmd.stdout)
        print(cmd.stderr)
        assert 0 == cmd.rc



from django.conf import settings
from django.http import JsonResponse, HttpResponse
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from enough.api.permissions import IsEnoughGroupMember
from enough.common import bind


@api_view(['POST'])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsEnoughGroupMember))
def delegate_test_dns(request):
    return JsonResponse(bind.delegate_dns(
        f'test.{settings.ENOUGH_DOMAIN}',
        request.data['name'],
        request.data['ip'],
    ), safe=False, status=201)


@api_view(['POST'])
def delegate_dns(request):
    return JsonResponse(bind.delegate_dns(
        f'd.{settings.ENOUGH_DOMAIN}',
        request.data['name'],
        request.data['ip'],
    ), safe=False, status=201)


@api_view(['GET'])
def ping(request):
    return HttpResponse('pong')



import website
testinfra_hosts = ['ansible://website-host']


def test_website(host):
    website.update(host)
    with host.sudo():
        cmd = host.run("""
        set -xe

        ! test -f /var/log/update-website.log.1.gz
        /usr/sbin/logrotate --force /etc/logrotate.d/website-logrotate
        test -f /var/log/update-website.log.1.gz
        test debian = $(stat --format=%U /var/log/update-website.log)
        """)
        print(cmd.stdout)
        print(cmd.stderr)
        assert 0 == cmd.rc

        host.run("apt-get install -y curl")

        assert host.run("curl -m 5 -I https://$(hostname -d)").rc == 0
        assert host.run("curl -m 5 -I https://www.$(hostname -d)").rc == 0



# -*- coding: utf-8 -*-

# Copyright (C) 2014-2015 Hartmut Goebel <h.goebel@crazy-compilers.com>
# Part of the DebOps - https://debops.org/

# This program is free software; you can redistribute
# it and/or modify it under the terms of the
# GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General
# Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# An on-line copy of the GNU General Public License can
# be downloaded from the FSF web page at:
# https://www.gnu.org/copyleft/gpl.html

from __future__ import print_function
from unittest import TestCase
from imp import reload
import os
import sys
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
try:
    from cStringIO import cStringIO
except ImportError:
    from io import StringIO as cStringIO
import tempfile
import shutil

import debops

__author__ = "Hartmut Goebel <h.goebel@crazy-compilers.com>"
__copyright__ = "Copyright 2014-2015 by Hartmut Goebel "
"<h.goebel@crazy-compilers.com>"
__licence__ = "GNU General Public License version 3 (GPL v3) or later"


def setenv(name, value):
    os.environ[name] = value


def unsetenv(name):
    os.environ[name] = ''


class TestConfigFilenames(TestCase):

    def test_get_config_filenames_no_env(self):
        unsetenv('XDG_CONFIG_HOME')
        unsetenv('XDG_CONFIG_DIRS')
        cfn = debops.config._get_config_filenames()
        self.assertListEqual(cfn,
                             ['/etc/debops.cfg',
                              '/etc/xdg/debops.cfg',
                              os.path.expanduser('~/.config/debops.cfg')])

    def test_get_config_filenames_with_XDG_CONFIG_HOME_set(self):
        setenv('XDG_CONFIG_HOME', '/myhome/mindy')
        unsetenv('XDG_CONFIG_DIRS')
        cfn = debops.config._get_config_filenames()
        self.assertListEqual(cfn,
                             ['/etc/debops.cfg',
                              '/etc/xdg/debops.cfg',
                              '/myhome/mindy/debops.cfg'])

    def test_get_config_filenames_with_XDG_CONFIG_DIRS_set(self):
        unsetenv('XDG_CONFIG_HOME')
        setenv('XDG_CONFIG_DIRS', '/tmp/mindy:/tmp/etc:/usr/local/etc')
        cfn = debops.config._get_config_filenames()
        self.assertListEqual(cfn,
                             ['/etc/debops.cfg',
                              '/usr/local/etc/debops.cfg',
                              '/tmp/etc/debops.cfg',
                              '/tmp/mindy/debops.cfg',
                              os.path.expanduser('~/.config/debops.cfg')])

    def test_get_config_filenames_with_XDG_vars_set(self):
        setenv('XDG_CONFIG_HOME', '/myhome/mindy')
        setenv('XDG_CONFIG_DIRS', '/tmp/etc:/usr/local/etc')
        cfn = debops.config._get_config_filenames()
        self.assertListEqual(cfn,
                             ['/etc/debops.cfg',
                              '/usr/local/etc/debops.cfg',
                              '/tmp/etc/debops.cfg',
                              '/myhome/mindy/debops.cfg'])


ANSIBLE_DEFAULTS = {'ansible_managed':
                    'This file is managed remotely, all changes will be lost'}


class TestReadConfig(TestCase):

    def setUp(self):
        self.sandbox = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.sandbox)
        self._saved_configfiles = debops.config._configfiles[:]
        # remove '/etc/debobs.cfg' to make results predictable
        debops.config._configfiles.remove('/etc/debops.cfg')

    def tearDown(self):
        debops.config._configfiles = self._saved_configfiles[:]

    def _make_configfile(self, dir, sect, *data):
        dir = os.path.join(self.sandbox, dir)
        os.makedirs(dir)
        fn = os.path.join(dir, 'debops.cfg')
        with open(fn, 'w') as fh:
            print("[{}]".format(sect), file=fh)
            for d in data:
                print(d, file=fh)
        return dir

    def _read_config(self, project_dir):
        # refresh debops._configfiles with set environment
        cfn = debops.config._get_config_filenames()
        cfn.remove('/etc/debops.cfg')
        debops.config._configfiles = cfn
        cfg = debops.config.read_config(project_dir)
        del cfg['paths']
        return cfg

    def test_read_config_files_simple(self):
        dirs = [self._make_configfile(dir, sect, data) for
                dir, sect, data in (
                    ['xdg_home', 'debops', 'home: /var/home'],
                    ['xdg_dir1', 'debops', 'name1: value1'],
                    ['xdg_dir2', 'debops', 'name2: value2'],
                )]
        setenv('XDG_CONFIG_HOME', dirs[0])
        setenv('XDG_CONFIG_DIRS', ':'.join(dirs[1:]))
        cfg = self._read_config('/non/existing/dir')
        self.assertDictEqual(cfg,
                             {'ansible defaults': ANSIBLE_DEFAULTS,
                              'debops': {'home': '/var/home',
                                         'name1': 'value1',
                                         'name2': 'value2'}
                              })

    def test_read_config_files_precedence(self):
        dirs = [self._make_configfile(dir, sect, data) for
                dir, sect, data in (
                    ['xdg_home', 'debops', 'home: /var/home'],
                    # xdg_dir1 has higher priority
                    ['xdg_dir1', 'debops', 'name1: value1'],
                    ['xdg_dir2', 'debops', 'name1: value2'],
                )]
        setenv('XDG_CONFIG_HOME', dirs[0])
        setenv('XDG_CONFIG_DIRS', ':'.join(dirs[1:]))
        cfg = self._read_config('/non/existing/dir')
        self.assertDictEqual(cfg,
                             {'ansible defaults': ANSIBLE_DEFAULTS,
                              'debops': {'home': '/var/home',
                                         'name1': 'value1'}
                              })

    def test_read_config_files_with_project_root(self):
        dirs = [self._make_configfile(dir, sect, data) for
                dir, sect, data in (
                    ['xdg_home', 'debops', 'home: /var/home'],
                    ['xdg_dir1', 'debops', 'name1: value1'],
                    ['proj_root', 'debops', 'name2: value2'],
                )]
        setenv('XDG_CONFIG_HOME', dirs[0])
        setenv('XDG_CONFIG_DIRS', dirs[1])
        os.rename(os.path.join(dirs[2], 'debops.cfg'),
                  os.path.join(dirs[2], '.debops.cfg'))
        cfg = self._read_config(dirs[2])
        self.assertDictEqual(cfg,
                             {'ansible defaults': ANSIBLE_DEFAULTS,
                              'debops': {'home': '/var/home',
                                         'name1': 'value1',
                                         'name2': 'value2'}
                              })

    def test_read_config_files_with_project_root_precedence(self):
        dirs = [self._make_configfile(dir, sect, *data) for
                dir, sect, data in (
                    ['xdg_home', 'debops', ('home: /var/home',)],
                    ['xdg_dir1', 'debops', ('name1: value1',)],
                    ['pro_root', 'debops', ('name1: value2',
                                            'home: /my/home')],
                )]
        setenv('XDG_CONFIG_HOME', dirs[0])
        setenv('XDG_CONFIG_DIRS', dirs[1])
        os.rename(os.path.join(dirs[2], 'debops.cfg'),
                  os.path.join(dirs[2], '.debops.cfg'))
        cfg = self._read_config(dirs[2])
        self.assertDictEqual(cfg,
                             {'ansible defaults': ANSIBLE_DEFAULTS,
                              'debops': {'home': '/my/home',
                                         'name1': 'value2'}
                              })


class TestReadConfig2(TestCase):

    def setUp(self):
        self.sandbox = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.sandbox)
        self._saved_configfiles = debops.config._configfiles[:]
        # remove '/etc/debobs.cfg' to make results predictable
        debops.config._configfiles.remove('/etc/debops.cfg')

    def tearDown(self):
        debops.config._configfiles = self._saved_configfiles[:]

    def _make_configfile(self, dir, sect, *data):
        dir = os.path.join(self.sandbox, dir)
        os.makedirs(dir)
        fn = os.path.join(dir, 'debops.cfg')
        with open(fn, 'w') as fh:
            print("[{}]".format(sect), file=fh)
            for d in data:
                print(d, file=fh)
        return dir

    def _read_config(self, project_dir):
        # refresh debops._configfiles with set environment
        cfn = debops.config._get_config_filenames()
        cfn.remove('/etc/debops.cfg')
        debops.config._configfiles = cfn
        return debops.config.read_config(project_dir)

    def test_defaults(self):
        dirs = [self._make_configfile(dir, sect, data) for
                dir, sect, data in (
                    ['xdg_home', 'xpaths', 'data-home: /opt/my/debops'],
                )]
        unsetenv('XDG_CONFIG_HOME')
        cfg = self._read_config('/non/existing/dir')
        self.assertDictEqual(
            cfg['paths'],
            {'data-home': os.path.expanduser('~/.local/share/debops'),
             'monorepo-path': os.path.expanduser(
                 '~/.local/share/debops/debops'),
             'install-path': os.path.expanduser(
                 '~/.local/share/debops/debops'),
             'playbooks-paths': [os.path.expanduser(
                 '~/.local/share/debops/debops/ansible/playbooks')],
             })

    def test_read_config_files_simple(self):
        dirs = [self._make_configfile(dir, sect, data) for
                dir, sect, data in (
                    ['xdg_home', 'paths', 'data-home: /opt/my/debops'],
                )]
        setenv('XDG_CONFIG_HOME', dirs[0])
        cfg = self._read_config('/non/existing/dir')
        self.assertDictEqual(
            cfg['paths'],
            {'data-home': '/opt/my/debops',
             'monorepo-path': '/opt/my/debops/debops',
             'install-path': '/opt/my/debops/debops',
             'playbooks-paths': ['/opt/my/debops/debops/ansible/playbooks'],
             })


class TestReadConfigDefaultsForPlattforms(TestCase):

    def setUp(self):
        self.platform = sys.platform

    def tearDown(self):
        sys.platform = self.platform

    def test_defaults_linux(self):
        sys.platform = 'linux2'
        reload(debops.config)
        cfgparser = configparser.SafeConfigParser()
        cfgparser.readfp(cStringIO(debops.config.DEFAULTS))
        self.assertEqual(cfgparser.get('paths', 'data-home'),
                         '$XDG_DATA_HOME/debops')

    def test_defaults_windows_without_APPDATA(self):
        sys.platform = 'win32'
        unsetenv('APPDATA')
        reload(debops.config)
        cfgparser = configparser.SafeConfigParser()
        cfgparser.readfp(cStringIO(debops.config.DEFAULTS))
        self.assertEqual(cfgparser.get('paths', 'data-home'),
                         '~\\Application Data/debops')

    def test_defaults_windows_with_APPDATA(self):
        sys.platform = 'win32'
        setenv('APPDATA', 'H:\\my\\own\\data')
        reload(debops.config)
        cfgparser = configparser.SafeConfigParser()
        cfgparser.readfp(cStringIO(debops.config.DEFAULTS))
        self.assertEqual(cfgparser.get('paths', 'data-home'),
                         'H:\\my\\own\\data/debops')

    def test_defaults_os_x(self):
        sys.platform = 'darwin'
        reload(debops.config)
        cfgparser = configparser.SafeConfigParser()
        cfgparser.readfp(cStringIO(debops.config.DEFAULTS))
        self.assertEqual(cfgparser.get('paths', 'data-home'),
                         '~/Library/Application Support/debops')



import time
import testinfra

from enough.common import retry

testinfra_hosts = ['ansible://postfix-host']


def test_sendmail(host):
    domain = host.run("hostname -d").stdout.strip()

    postfix_host = host
    postfix_client_host = testinfra.host.Host.get_host(
        'ansible://bind-host',
        ansible_inventory=host.backend.ansible_inventory)

    cmd = postfix_client_host.run("""
    ( echo 'To: loic+boundtofail@dachary.org' ; echo POSTFIX TEST ) |
    /usr/sbin/sendmail -v -F 'NO REPLY' -f 'noreply@{}' -t
    """.format(domain))
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc

    check = ("grep -q 'TLS connection established to postfix-host' "
             "/var/log/syslog")
    for _ in range(300):
        print(check)
        cmd = postfix_client_host.run(check)
        if cmd.rc == 0:
            break
        time.sleep(1)

    @retry.retry(AssertionError, tries=8)
    def wait_for_mail():
        with postfix_host.sudo():
            cmd = host.run("""
            ls /var/spool/postfix/hold
            grep -q 'POSTFIX TEST' /var/spool/postfix/hold/*
            """)
        print(cmd.stdout)
        assert cmd.rc == 0, f'{cmd.stdout} {cmd.stderr}'
    wait_for_mail()


def test_encryption(host):
    with host.sudo():
        cmd = host.run("""
        set -ex
        postsuper -d ALL
        ( echo Subject: encrypted ; echo ; echo ENCRYPTED CONTENT ) | \
          mail -r debian@localhost debian@localhost
        for d in 0 1 2 4 ; do
           sleep $d
           grep -q 'PGP MESSAGE' /var/spool/postfix/hold/* || continue
           grep -q 'ENCRYPTED CONTENT' /var/spool/postfix/hold/* && exit 1
           ls /var/spool/postfix/hold | while read m ; do
              postcat -q $m | sudo -u zeyple gpg --homedir /var/lib/zeyple/keys --decrypt | \
                grep -q 'ENCRYPTED CONTENT' && echo FOUND && break
           done
        done
        """)
        print(cmd.stdout)
        print(cmd.stderr)
        assert 0 == cmd.rc
        assert 'FOUND' in cmd.stdout

        cmd = host.run("""
        set -ex
        postsuper -d ALL
        ( echo Subject: encrypted ; echo ; echo CLEAR TEXT ) | \
          mail -r debian@localhost debian+notencrypted@localhost
        for d in 0 1 2 4 ; do
           sleep $d
           grep -q 'CLEAR TEXT' /var/spool/postfix/hold/* && echo FOUND && break
        done
        """)
        print(cmd.stdout)
        print(cmd.stderr)
        assert 0 == cmd.rc
        assert 'FOUND' in cmd.stdout



import re

from bs4 import BeautifulSoup
from tests.icinga_helper import IcingaHelper
from enough.common import retry
import requests
import testinfra
import yaml

testinfra_hosts = ['ansible://icinga-host']


class TestChecks(IcingaHelper):

    def test_icinga_host(self):
        hosts = self.get_hosts()
        assert 'bind-host' in hosts
        assert 'deleted-host' not in hosts

    def test_icinga_version(self, pytestconfig):
        url = f'{self.get_api_url()}/status/IcingaApplication'
        answer = requests.get(url, **self.connection_params())
        answer.raise_for_status()
        results = answer.json().get('results', [])
        assert len(results) == 1
        version = results[0]['status']['icingaapplication']['app']['version']

        host = testinfra.get_host(f'ansible://{TestChecks.icinga_host}',
                                  ansible_inventory=self.inventory)
        host_vars = host.ansible.get_variables()
        expected_version = host_vars.get('icinga_version')
        if not expected_version:
            icinga2_common_defaults = 'playbooks/icinga/roles/icinga2_common/defaults/main.yml'
            expected_version = yaml.safe_load(open(icinga2_common_defaults))['icinga_version']

        expected_version = expected_version.rsplit('.', 1)[0]
        assert re.search(expected_version, version)

    def test_disk(self):
        assert self.is_service_ok('website-host!disk')
        assert self.is_service_ok('packages-host!disk')

    def test_icinga_ntp_time(self):
        assert self.is_service_ok('website-host!systemd-timesyncd is working')

    def test_memory(self):
        assert self.is_service_ok('icinga-host!memory')
        assert self.is_service_ok('website-host!memory')

    def test_grafana(self):
        (s, a) = self.get_web_session()

        @retry.retry(AssertionError, tries=8)
        def has_image():
            r = s.get(
                f'https://{a}/icingaweb2/monitoring/service/show?host=icinga-host&service=load')
            r.raise_for_status()
            soup = BeautifulSoup(r.text, 'html.parser')
            module_selector = '.module-grafana'
            assert len(soup.select(module_selector)) > 0, f"select('{module_selector}') not found"
            img_selector = '.grafana-img'
            img = soup.select(img_selector)
            assert len(img) > 0, f"select('{img_selector}') not found"
            img = img[0]
            assert hasattr(img, 'width'), f"select('{img_selector}') not width attribute"
            return True

        assert has_image()



import datetime
import os

import pytest
import requests_mock as requests_mock_module

from tests.modified_environ import modified_environ

from enough import settings
from enough.common import openstack
from enough.common.openstack import Stack, Heat, OpenStack


#
# Stack
#
@pytest.mark.openstack_integration
def test_stack_create_or_update(openstack_name, openstack_variables,
                                dot_openstack):
    name = openstack_name
    network = openstack_name
    class_c = '10.101.30'
    cidr = f'{class_c}.0/24'
    d = Heat.hostvars_to_stack(openstack_name, openstack_variables)
    d.update(
        internal_network=network,
        internal_network_cidr=cidr
    )
    s = Stack(settings.CONFIG_DIR, d)
    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
    r = s.create_or_update()
    assert r['port'] == '22'
    ipv4 = r['ipv4']
    o = OpenStack(settings.CONFIG_DIR)
    assert o.server_connected_to_network(name, network)
    assert r == s.create_or_update()
    assert o.server_connected_to_network(name, network)
    assert r['ipv4'] == ipv4
    s.delete()
    assert not o.network_exists(network)


#
# Heat
#
def test_heat_definition():
    h = Heat(settings.CONFIG_DIR)
    definitions = h.get_stack_definitions()
    assert 'bind-host' in definitions
    definition = h.get_stack_definition('bind-host')
    assert definition['name'] == 'bind-host'


def test_host_from_volume():
    h = Heat(settings.CONFIG_DIR)
    assert h.host_from_volume('cloud-volume') == 'cloud-host'
    assert h.host_from_volume('unknown-volume') is None


def test_heat_definition_encrypted():
    d = 'tests/enough/common/test_openstack/config_dir'
    h = Heat(d)
    definitions = h.get_stack_definitions(share_dir=d)
    assert 'my-host' in definitions
    assert definitions['my-host']['myvariable'] == 'myvalue'


def test_create_test_subdomain_no_bind(mocker):
    mocker.patch('os.path.exists', return_value=True)
    mocker.patch('enough.common.openstack.Stack.list', return_value=[])
    h = Heat(settings.CONFIG_DIR)
    assert h.create_test_subdomain('my.tld') is None


def test_create_test_subdomain_with_bind(tmpdir, mocker, requests_mock):
    mocker.patch('enough.settings.CONFIG_DIR', str(tmpdir))
    d = f'{tmpdir}/inventory/group_vars/all'
    os.makedirs(d)
    assert not os.path.exists(f'{d}/domain.yml')
    mocker.patch('os.path.exists', return_value=True)
    mocker.patch('enough.common.openstack.Stack.list', return_value=['bind-host'])
    mocker.patch('enough.common.openstack.Stack.set_public_key')
    mocker.patch('enough.common.openstack.Stack.create_or_update', return_value={
        'ipv4': '1.2.3.4',
    })
    mocker.patch('enough.common.openstack.Heat.get_stack_definition')
    requests_mock.post(requests_mock_module.ANY, status_code=201)
    h = Heat(settings.CONFIG_DIR)
    with modified_environ(ENOUGH_API_TOKEN="TOKEN"):
        fqdn = h.create_test_subdomain('my.tld')
    assert '.test.my.tld' in fqdn
    assert os.path.exists(f'{d}/domain.yml')


#
# OpenStack
#
def test_server_ip_in_network_ok(mocker):
    ip = '1.2.3.4'
    mocker.patch('enough.common.openstack.OpenStack.server_port_list',
                 return_value="[{'ip_address': '" + ip + "'}, {'ip_address': '1:2:3:4:5'}]")
    o = OpenStack(settings.CONFIG_DIR)
    assert o.server_ip_in_network('server', 'network') == ip


def test_server_ip_in_network_none(mocker):
    mocker.patch('enough.common.openstack.OpenStack.server_port_list',
                 return_value='[]')
    o = OpenStack(settings.CONFIG_DIR)
    assert o.server_ip_in_network('server', 'network') is None


@pytest.mark.openstack_integration
def test_network(openstack_name, dot_openstack):
    o = OpenStack(settings.CONFIG_DIR)
    o.network_and_subnet_create(openstack_name, '10.11.12.0/24')
    assert o.network_exists(openstack_name)
    assert o.subnet_exists(openstack_name)

    dns_ip = '1.2.3.4'
    o.subnet_update_dns(openstack_name, dns_ip)
    r = o.o.subnet.show('--format=value', '-c', 'dns_nameservers', openstack_name)
    assert r.strip() == f"['{dns_ip}']"


@pytest.mark.openstack_integration
def test_backup_create_with_name(openstack_name, dot_openstack, caplog):
    o = OpenStack(settings.CONFIG_DIR)
    o.o.volume.create('--size=1', openstack_name)
    assert o.backup_create([openstack_name]) == 1
    assert o.backup_create([openstack_name]) == 0
    available_snapshot = f'AVAILABLE {o.backup_date()}-{openstack_name}'
    assert available_snapshot in caplog.text
    assert o.backup_prune(0) == 1
    assert o.backup_prune(0) == 0


@pytest.mark.openstack_integration
def test_volume_prune(openstack_name, dot_openstack, caplog):
    o = OpenStack(settings.CONFIG_DIR)

    volume_precious = openstack_name
    o.o.volume.create('--size=1', volume_precious)
    days = 2

    old = days + 1
    old_date = (datetime.datetime.today() - datetime.timedelta(old)).strftime('%Y-%m-%d')

    old_volume_with_snapshot = f'{old_date}-old-with-snapshot-{openstack_name}'
    o.o.volume.create('--size=1', old_volume_with_snapshot)
    old_volume_snapshot = f'snapshot-{openstack_name}'
    o.o.volume.snapshot.create('--volume', old_volume_with_snapshot, old_volume_snapshot)

    old_volume = f'{old_date}-old-{openstack_name}'
    o.o.volume.create('--size=1', old_volume)

    recent = days - 1
    recent_date = (datetime.datetime.today() - datetime.timedelta(recent)).strftime('%Y-%m-%d')

    recent_volume = f'{recent_date}-{openstack_name}'
    o.o.volume.create('--size=1', recent_volume)

    assert o.volume_prune(days) == {'deleted': [old_volume],
                                    'has_snapshots': [old_volume_with_snapshot],
                                    'no_date_prefix': [volume_precious],
                                    'recent': [recent_volume]}


@pytest.mark.openstack_integration
def test_backup_create_no_names(openstack_name, dot_openstack, caplog):
    o = OpenStack(settings.CONFIG_DIR)
    o.o.volume.create('--size=1', openstack_name)
    o.backup_create([])
    available_snapshot = f'AVAILABLE {o.backup_date()}-{openstack_name}'
    assert available_snapshot in caplog.text


@pytest.mark.openstack_integration
def test_openstack_replace_volume(openstack_name, dot_openstack,
                                  openstack_variables):
    d = Heat.hostvars_to_stack(openstack_name, openstack_variables)
    d.update({
        'volumes': [
            {
                'size': '1',
                'name': openstack_name,
            },
        ],
    })
    s = Stack(settings.CONFIG_DIR, d)
    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
    s.create_or_update()

    o = OpenStack(settings.CONFIG_DIR)
    assert openstack_name in o.o.volume.list('--name', openstack_name)

    other_volume = f'{openstack_name}_other'
    date = o.backup_date()
    backup_volume = f'{date}-{openstack_name}'

    o.o.volume.create('--size=1', other_volume)
    o.replace_volume(openstack_name, other_volume, delete_volume=True)
    assert o.o.volume.list('--name', other_volume).strip() == ''
    assert o.o.volume.list('--name', backup_volume).strip() == ''
    assert o.o.volume.list(
        '-f=value', '-c=Name', '--name', openstack_name).strip() == openstack_name

    other_volume = f'{openstack_name}_backedup'
    o.o.volume.create('--size=1', other_volume)
    o.replace_volume(openstack_name, other_volume, delete_volume=False)
    assert o.o.volume.list('--name', other_volume).strip() == ''
    assert o.o.volume.list(
        '-f=value', '-c=Name', '--name', backup_volume).strip() == backup_volume
    assert o.o.volume.list(
        '-f=value', '-c=Name', '--name', openstack_name).strip() == openstack_name


@pytest.mark.openstack_integration
def test_openstack_volume_resize_ok(openstack_name, dot_openstack,
                                    openstack_variables):
    size = 1
    d = Heat.hostvars_to_stack(openstack_name, openstack_variables)
    d.update({
        'volumes': [
            {
                'size': str(size),
                'name': openstack_name,
            },
        ],
    })
    s = Stack(settings.CONFIG_DIR, d)
    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
    s.create_or_update()

    o = OpenStack(settings.CONFIG_DIR)

    assert size == int(o.o.volume.show(
        '-c', 'size', '--format=value', openstack_name).strip())

    new_size = 2
    assert o.volume_resize(openstack_name, openstack_name, new_size) is True

    assert new_size == int(o.o.volume.show(
        '-c', 'size', '--format=value', openstack_name).strip())

    assert o.volume_resize(openstack_name, openstack_name, new_size) is False

    with pytest.raises(openstack.OpenStackVolumeResizeMismatch):
        o.volume_resize(openstack_name, 'UNKNOWN', new_size)

    with pytest.raises(openstack.OpenStackVolumeResizeShrink):
        o.volume_resize(openstack_name, openstack_name, 1)


@pytest.mark.openstack_integration
def test_openstack_volume_resize_no_volume(openstack_name, dot_openstack, openstack_variables):
    d = Heat.hostvars_to_stack(openstack_name, openstack_variables)
    s = Stack(settings.CONFIG_DIR, d)
    s.set_public_key(f'{settings.CONFIG_DIR}/infrastructure_key.pub')
    s.create_or_update()

    o = OpenStack(settings.CONFIG_DIR)

    with pytest.raises(openstack.OpenStackVolumeResizeMissing):
        o.volume_resize(openstack_name, openstack_name, 1)


@pytest.mark.openstack_integration
def test_openstack_security_group(openstack_name, dot_openstack):
    o = OpenStack(settings.CONFIG_DIR)
    o.o.security.group.create(openstack_name)
    assert o.delete_security_group(openstack_name) is True
    assert o.delete_security_group(openstack_name) is False


@pytest.mark.openstack_integration
def test_destroy_volumes_with_same_name(openstack_name, dot_openstack):
    volume_name = f'{openstack_name}-test-destroy-volume'
    o = OpenStack(settings.CONFIG_DIR)
    o.o.volume.create('--size', '1', volume_name)
    o.o.volume.create('--size', '1', volume_name)
    assert 2 == o.o.volume.list().count(volume_name)
    o.destroy_everything(openstack_name)
    assert 0 == o.o.volume.list().count(volume_name)



testinfra_hosts = ['ansible://packages-host']


def test_enough_android(host):
    cmd = host.run("""
    set -xe
    flock /tmp/update-packages \
          bash -x /srv/enough-android-update-packages.sh
    test -f /usr/share/nginx/html/enough.apk
    """)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc



import json
import logging
import re
import sh
import socket

from django.conf import settings


def delegate_dns(zone, name, ip):
    results = []
    results.append(nsupdate({
        "zone": f"{zone}.",
        "record": f"ns-{name}.{zone}.",
        "ttl": "1800",
        "type": "A",
        "value": ip,
    }, state='present'))
    #
    # For NS records, "present" is not idempotent, but "absent" is
    # idempotent so make sure it is absent before adding it
    #
    results.append(nsupdate({
        "zone": f"{zone}.",
        "record": f"{name}.{zone}.",
        "ttl": "1800",
        "type": "NS",
        "value": f"ns-{name}.{zone}.",
    }, state='absent'))
    results.append(nsupdate({
        "zone": f"{zone}.",
        "record": f"{name}.{zone}.",
        "ttl": "1800",
        "type": "NS",
        "value": f"ns-{name}.{zone}.",
    }, state='present'))
    return results


def nsupdate(data, state):
    configdir = settings.CONFIG_DIR
    bind_host = data.get('bind_host', 'bind-host')
    bind_ip = socket.gethostbyname(bind_host)
    args = [
        f'server={bind_ip}',
        f'state={state}',
    ]
    for k in ('zone', 'record', 'ttl', 'type', 'value'):
        if k in data:
            args.append(f'{k}={data[k]}')
    logger = logging.getLogger(__name__)
    r = sh.ansible(
        '-i', f'{bind_host},',
        '--private-key', f'{configdir}/infrastructure_key',
        '--user=debian',
        bind_host,
        '--one-line',
        f'--playbook-dir={configdir}',
        '-m', 'nsupdate', '-a', " ".join(args),
        _tee=True,
        _truncate_exc=False,
        _out=lambda x: logger.info(x.strip()),
        _err=lambda x: logger.info(x.strip()),
        _env={
            'ANSIBLE_NOCOLOR': 'true',
            'ANSIBLE_HOST_KEY_CHECKING': 'False',
        })
    json_result = re.sub(r'.*?=> ', '', r.stdout.decode('utf-8'))
    print(json_result)
    return json.loads(json_result)



def pytest_addoption(parser):
    parser.addoption(
        "--enough-hosts",
        action="store",

        default="bind-host,website-host",
        help="list of hosts"
    )
    parser.addoption(
        "--enough-service",
        action="store",
        default="pad",
        help="service"
    )



import os

from tests.modified_environ import modified_environ
from enough import cmd


def test_remap():
    p = '.enough/thing'
    unchanged = '/foo/bar/'
    expected = [f'{os.path.expanduser("~")}/{p}', unchanged]
    assert cmd.EnoughApp.remap([f'/home/user/{p}', unchanged]) == expected
    expected = [f'--option={os.path.expanduser("~")}/{p}', unchanged]
    assert cmd.EnoughApp.remap([f'--option=/home/user/{p}', unchanged]) == expected


def test_preserve_ownership(mocker, tmpdir):
    #
    # not running as root, do nothing
    #
    assert cmd.EnoughApp.preserve_ownership() is False

    #
    # pretend to run as root and chown to self
    #
    mocker.patch('os.geteuid', return_value=0)
    mocker.patch('os.path.expanduser', return_value=str(tmpdir))
    assert cmd.EnoughApp.preserve_ownership() is True


def test_playbook(capsys, mocker):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.ansible_utils.Playbook.run_from_cli',
                 side_effect=lambda **kwargs: print('PLAYBOOK'))
    assert cmd.main(['playbook']) == 0
    out, err = capsys.readouterr()
    assert 'PLAYBOOK' in out


def test_openstack(capsys, mocker):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.openstack.OpenStack.run',
                 side_effect=lambda *args, **kwargs: print('OPENSTACK'))
    with modified_environ(OS_CLIENT_CONFIG_FILE="/dev/null"):
        assert cmd.main(['openstack', '--debug', 'help']) == 0
    out, err = capsys.readouterr()
    assert 'OPENSTACK' in out



import subprocess
import sys

from enough.common import dotenough


class SSH(object):

    def __init__(self, config_dir, **kwargs):
        self.config_dir = config_dir
        self.args = kwargs
        self.dot = dotenough.DotEnoughOpenStack(self.config_dir,
                                                self.args['domain'])
        self.dot.ensure()

    def ssh(self, host, args, interactive=True):
        hosts = dotenough.Hosts(self.config_dir)
        ip = hosts.get_ip(host)
        if not ip:
            raise Exception(f'{host} is not found in {hosts.f}')
        ssh = ['ssh', '-oStrictHostKeyChecking=no']
        port = hosts.get_port(host)
        if port != '22':
            ssh.append(f'-p{port}')
        ssh.append(f'-i{self.dot.private_key()}')
        ssh.append(f'debian@{ip}')
        if interactive:
            kwargs = dict(stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr)
        else:
            kwargs = dict(capture_output=True)
        return subprocess.run(ssh + args, **kwargs)



import pytest
from tests.infrastructure import get_driver

testinfra_hosts = ['ansible://bind-host', 'ansible://bind-client-host']


def test_resolvconf(host):
    if get_driver() == 'docker':
        pytest.skip("docker does not do dhcp")
    resolvconf_before = host.run("cat /etc/resolv.conf").stdout.strip()
    with host.sudo():
        cmd = host.run("ifdown eth0 ; ifup eth0 ; ifdown eth1 ; ifup eth1")
    assert 0 == cmd.rc
    resolvconf_after = host.run("cat /etc/resolv.conf").stdout.strip()
    assert resolvconf_before == resolvconf_after



testinfra_hosts = ['ansible://packages-host']


def test_packages(host):
    cmd = host.run("""
    set -xe

    ! test -f /var/log/update-packages.log.1.gz
    echo >> /var/log/update-packages.log
    sudo /usr/sbin/logrotate --force /etc/logrotate.d/packages-logrotate
    test -f /var/log/update-packages.log.1.gz
    test debian = $(stat --format=%U /var/log/update-packages.log)
    """)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc


def test_displayed_packages(host):
    with host.sudo():
        host.run("apt-get install -y curl")

    assert host.run("curl -s -m 5 https://packages.$(hostname -d)").rc == 0



testinfra_hosts = ['ansible://cloud-host']


def test_nextcloud(host):
    cmd = host.run("""
    set -xe
    mount | grep /dev/mapper/spare
    curl --silent https://cloud.$(hostname -d)/login | \
         grep --quiet 'This application requires JavaScript'
    """)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc


def test_nextcloud_via_tor(host):
    cmd = host.run("""
    set -xe
    hostname=$(sudo cat /var/lib/tor/services/cloud/hostname)
    torsocks curl --silent http://$hostname/login | \
             grep --quiet 'This application requires JavaScript'
    """)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc



import sh

testinfra_hosts = ['ansible://authorized-keys-host']


def test_all(host):
    address = host.ansible.get_variables()['ansible_host']
    marker = "MARKER"
    key = 'playbooks/authorized_keys/roles/authorized_keys/files/test_keys/testkey'
    sh.chmod('600', key)
    r = sh.ssh('-oStrictHostKeyChecking=no', '-i', key, 'debian@' + address, 'echo', marker)
    assert r.stdout.decode('utf-8').strip() == marker


def test_remove(host):
    assert host.file("/home/debian/.ssh/authorized_keys").contains("cigale.amie.coop")
    assert not host.file("/home/debian/.ssh/authorized_keys").contains("MBPG.local")



# (c) 2015, Hartmut Goebel <h.goebel@crazy-compilers.com>
# Based on `runner/lookup_plugins/items.py` for Ansible
#   (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
#
# This file is part of Debops.
# This file is NOT part of Ansible yet.
#
# Debops is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Debops.  If not, see <https://www.gnu.org/licenses/>.
'''

This file implements the `with_lists` lookup filter for Ansible. In
differenceto `with_items`, this one does *not* flatten the lists passed to.

Example:

  - debug: msg="{{item.0}} -- {{item.1}} -- {{item.2}}"
    with_lists:
     - ["General", "Verbosity", "0"]
     - ["Mapping", "Nobody-User", "nobody"]
     - ["Mapping", "Nobody-Group", "nogroup"]

Output (shortend):
    "msg": "General -- Verbosity -- 0"
    "msg": "Mapping -- Nobody-User -- nobody"
    "msg": "Mapping -- Nobody-Group -- nogroup"
'''

import ansible.utils as utils
import ansible.errors as errors

try:
    from ansible.plugins.lookup import LookupBase
except ImportError:
    LookupBase = object


class LookupModule(LookupBase):

    def __init__(self, basedir=None, **kwargs):
        self.basedir = basedir

    def run(self, terms, inject=None, **kwargs):
        terms = utils.listify_lookup_plugin_terms(terms, self.basedir, inject)

        if not isinstance(terms, (list, set)):
            raise errors.AnsibleError("with_list expects a list or a set")

        for i, elem in enumerate(terms):
            if not isinstance(elem, (list, tuple)):
                raise errors.AnsibleError(
                        "with_list expects a list (or a set) of lists"
                        " or tuples, but elem %i is not")

        return terms



testinfra_hosts = ['ansible://bind-host']


#
# Enough is installed on bind-host using the script from packages-host
#
def test_enough_pip(host):
    assert host.file("/usr/local/bin/enough-build-docker-image.sh").exists
    cmd = host.run("""
    set -ex
    docker run --rm --entrypoint enough enough --help > /tmp/e
    cat /tmp/e
    grep -q 'service create' /tmp/e
    """)
    print(cmd.stdout)
    print(cmd.stderr)
    assert 0 == cmd.rc



testinfra_hosts = ['ansible://owncanotweb-host']


def test_ownca_files_present(host):
    host.file("/etc/ssl/certs/enough.pem").exists
    domain = host.run("hostname -d").stdout.strip()
    host.file(f"/etc/certificates/{domain}.crt").exists



from tests.icinga_helper import IcingaHelper

testinfra_hosts = ['ansible://bind-host']

IcingaHelper.icinga_host = 'bind-host'


class TestChecks(IcingaHelper):

    def test_host(self):
        assert 'psono-host' in self.get_hosts(host='psono-host')

    def test_service(self, host):
        assert self.is_service_ok('psono-host!psono')



from enough import cmd


def test_backup_restore(mocker, tmp_config_dir):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.Enough.restore_local')
    assert cmd.main(['--debug', 'backup', 'restore', 'name']) == 0


def test_backup_clone_volume(mocker, tmp_config_dir):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.Enough.cli_clone_volume')
    assert cmd.main(['--debug', 'backup', 'clone', 'volume', 'name']) == 0



import sys

from enough import cmd


def main(argv=sys.argv[1:]):
    cmd.EnoughApp.set_enough_domain(argv)
    myapp = cmd.EnoughApp('enough.internal.cli')
    return myapp.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



from enough import cmd


def test_volume_resize(mocker, tmp_config_dir):
    # do not tamper with logging streams to avoid
    # ValueError: I/O operation on closed file.
    mocker.patch('cliff.app.App.configure_logging')
    mocker.patch('enough.common.Enough.volume_resize')
    assert cmd.main(['volume', 'resize', 'HOST', 'VOLUME']) == 0
